<?php

class GTCBase {

    CONST STOCK_UPDATE_REASON_BACKGROUND = 'background_sync';
    CONST STOCK_UPDATE_REASON_VIEW = 'product_view';
    CONST STOCK_UPDATE_KEY = 'gtc-stock-updated-date';
    CONST STOCK_UPDATE_FAILED_KEY = '_gtc-stock-update-failed-date';
    CONST STOCK_UPDATE_REASON_KEY = '_gtc-stock-update-reason';

    protected function getLogger(){

        return wc_get_logger();
    }

    protected function getClient(){

        return new GTCSyncClient(
            get_option(GTCSYNC_PREFIX.'subdomain'),
            get_option(GTCSYNC_PREFIX.'username'),
            get_option(GTCSYNC_PREFIX.'password')
        );
    }

    protected function isConfigured(){

        return $this->getClient()->hasCredentials();
    }

    protected function acceptedEula(){
        return empty(get_option(GTCSYNC_PREFIX.'eula_version')) == false;
    }

    protected function handleWpError($error){

        if (is_wp_error($error) == false)
            return $error;

        throw new Exception($error->get_error_message());
    }

    protected function hasCachePlugin()
    {
        $plugins = get_option('active_plugins');
        $cache_plugins = array_filter($plugins, function($plugin_path){

            $plugin_path = strtolower($plugin_path);

            // Check for 'cache' in name
            if (strpos($plugin_path, 'cache') !== false)
                return true;

            // Specific plugins
            if (strpos($plugin_path, 'wp-rocket') !== false)
                return true;

            return false;
        });

        return empty($cache_plugins) == false;
    }

    protected function logError($exception, $metadata = [], $severity = 'error'){

        $str = $exception->getMessage().' - '.json_encode($metadata);

        if ($severity == 'error')
            $this->getLogger()->error($str);
        else
            $this->getLogger()->info($str);

        if ($this->acceptedEula()) {
            $bugsnag = new GTCBugsnag(GTCSYNC_BUGSNAG_KEY);
            $bugsnag->notify(
                $exception,
                $metadata,
                [
                    'id' => get_option(GTCSYNC_PREFIX . 'subdomain'),
                    'gtc_client_subdomain' => get_option(GTCSYNC_PREFIX . 'subdomain'),
                    'gtc_outlet_id' => get_option(GTCSYNC_PREFIX . 'outlet_id'),
                    'gtc_outlet_name' => get_option(GTCSYNC_PREFIX . 'outlet_name'),
                    'gtc_username' => get_option(GTCSYNC_PREFIX . 'username'),
                    'site_url' => get_site_url(),
                ],
                $severity,
                GTCSYNC_BUGSNAG_RELEASE_STAGE
            );
        }
    }

    protected function woocommerceActive(){

        return class_exists('WooCommerce');
    }

    protected function hasTaxClasses(){

        $value = get_option(GTCSYNC_PREFIX.'vat_rates');
        return empty($value) == false;
    }

    protected function setTimeZone($date, $format = null){

        $format = $format ? $format : GTCSYNC_DATEFORMAT;

        return DateTime::createFromFormat(
                GTCSYNC_DATEFORMAT,
                $date
            )
            ->setTimeZone(new DateTimeZone('Europe/London'))
            ->format($format);
    }

    /**
     * Convert inventory data from GTC to woocommerce.
     */
    protected function convertInventoryData($track_inventory, $quantity){

        // Count can be integer or null, need to check both cases
        $count = $quantity;
        $count_value = intval($count);

        // Will always be 0 or 1
        $track_inventory = (bool)$track_inventory;

        // For variants, track_inventory can be true and
        // count is null. To prevent woocommerce marking
        // the product OOS, set to not track.
        if ($count == null)
            $track_inventory = false;

        return [
            'track_inventory' => $track_inventory,
            'quantity' => $count_value
        ];
    }

    protected function getSystemInfo(){

        $data = [];

        // Versions
        $data['plugin_version'] = GTCSYNC_VERSION;
        $data['php_version'] = sprintf('%s.%s.%s', PHP_MAJOR_VERSION, PHP_MINOR_VERSION, PHP_RELEASE_VERSION);
        $data['wordpress_version'] = get_bloginfo('version');

        $wc_version = '';
        try {
            $wc_version = WooCommerce::instance()->version;
        } catch (Throwable $e) { }
        $data['woocommerce_version'] = $wc_version;

        $cron_constants = [
            'DISABLE_WP_CRON',
            'ALTERNATE_WP_CRON',
            'WP_CRON_LOCK_TIMEOUT',
        ];

        // Cron
        $data['cron'] = [];
        foreach ($cron_constants as $constant){
            $value = '[UNDEFINED]';

            if (defined($constant))
                $value = constant($constant);

            $data['cron'][$constant] = $value;
        }

        $data['cron']['wp-cron_script_present'] = file_exists(ABSPATH.'/wp-cron.php');

        // Timezone
        $data['timezone'] = [
            'wp_timezone_string' => get_option('timezone_string'),
            'wp_gmt_offset' => get_option('gmt_offset'),
            'php_default_timezone' => date_default_timezone_get(),
        ];

        return $data;
    }

    protected function getActivePlugins(){

        $all_plugins = get_plugins();
        $active_plugins = get_option('active_plugins');

        return array_reduce($active_plugins, function ($carry, $plugin_path) use ($all_plugins) {
            if (isset($all_plugins[$plugin_path])) {
                $plugin_details = $all_plugins[$plugin_path];
                $carry[$plugin_details['Name']] = $plugin_details['Version'];
            }
            return $carry;
        }, []);
    }

    protected function joinPaths(){

        $paths = [];

        foreach (func_get_args() as $arg) {
            $paths[] = trim($arg, DIRECTORY_SEPARATOR);
        }

        $paths = array_filter($paths);

        return DIRECTORY_SEPARATOR . join(DIRECTORY_SEPARATOR, $paths);
    }

    protected function supportCachePlugins(){
        return [
            'W3 Total Cache',
            'WP Super Cache',
        ];
    }

    protected function clearProductCache($product_id){

        try {
            // W3 Total Cache
            if (function_exists('w3tc_flush_post'))
                w3tc_flush_post($product_id);

            // WP Super Cache
            // See: https://odd.blog/wp-super-cache-developers/
            if (function_exists('wp_cache_post_change')){
                $old_val = $GLOBALS["super_cache_enabled"];
                $GLOBALS["super_cache_enabled"] = 1;
                wp_cache_post_change($product_id);
                $GLOBALS["super_cache_enabled"] = $old_val;
            }

        } catch (Throwable $e) {
            $e = new Exception('Error flushing product page cache', 500, $e);
            $this->logError($e);
        }
    }

    protected function clearAllCache(){

        try {
            // W3 Total Cache
            if (function_exists('w3tc_flush_all'))
                w3tc_flush_all();

            // WP Super Cache
            if (function_exists('wp_cache_clear_cache'))
                wp_cache_clear_cache();

        } catch (Throwable $e) {
            $e = new Exception('Error flushing all cached pages', 500, $e);
            $this->logError($e);
        }
    }

    protected function htmlToPlainText($input){
        $out = str_ireplace(["<br />","<br>","<br/>"], "\n", $input);
        $out = wp_strip_all_tags($out);
        $out = html_entity_decode($out);
        return $out;
    }

    protected function getProductCount($status = 'any')
    {
        $args = [
            'post_type' => 'product',
            'post_status' => $status,
        ];

        $query = new WP_Query($args);
        return intval($query->found_posts);
    }

    public function getStockUpdateInfo(WC_Product $product)
    {
        // Get dates of last successful and last failed stock updates
        $stock_success_date_str = $product->get_meta(static::STOCK_UPDATE_KEY);
        $stock_success_date = null;
        if (empty($stock_success_date_str) == false)
            $stock_success_date = DateTime::createFromFormat(GTCSYNC_DATEFORMAT, $stock_success_date_str);

        $stock_failed_date_str = $product->get_meta(static::STOCK_UPDATE_FAILED_KEY);
        $stock_failed_date = null;
        if (empty($stock_failed_date_str) == false)
            $stock_failed_date = DateTime::createFromFormat(GTCSYNC_DATEFORMAT, $stock_failed_date_str);

        // Get last checked time and status
        $stock_update_date = null;
        if ($stock_success_date && $stock_failed_date)
            $stock_update_date = $stock_success_date > $stock_failed_date ? $stock_success_date : $stock_failed_date;
        else if ($stock_success_date)
            $stock_update_date = $stock_success_date;
        else if ($stock_failed_date)
            $stock_update_date = $stock_failed_date;

        // Check for success
        $is_success = $stock_success_date && $stock_update_date == $stock_success_date;

        // Get cause of last update
        $stock_reason_text = null;
        $stock_reason = null;
        if ($product->meta_exists(static::STOCK_UPDATE_REASON_KEY))
            $stock_reason = $product->get_meta(static::STOCK_UPDATE_REASON_KEY);

        $stock_reasons = [
            static::STOCK_UPDATE_REASON_BACKGROUND => 'background sync',
            static::STOCK_UPDATE_REASON_VIEW => 'product viewed by user',
        ];

        $last_update_type = null;
        if (isset($stock_reasons[$stock_reason]))
            $last_update_type = $stock_reasons[$stock_reason];

        // Build text
        $text = 'Never updated';
        if ($stock_update_date != null){
            $msg = $is_success ? 'Stock updated' : 'Stock update failed';
            $text = trim(sprintf('%s %s ago', $msg, human_time_diff($stock_update_date->format('U'))));

            if ($last_update_type)
                $text .= ' (' . $stock_reasons[$stock_reason] . ')';
        }

        return [
            'text' => $text,
            'last_update_successful' => $is_success,
            'last_success' => $stock_success_date,
            'last_failure' => $stock_failed_date,
            'last' => $stock_update_date,
            'last_update_type' => $last_update_type,
        ];
    }

    public function skuHtml($sku)
    {
        $sku_str = empty($sku) ? '<span class="gtc-sku-empty">EMPTY</span>' : htmlspecialchars($sku);
        return sprintf('<span class="gtc-sku">%s</span>', $sku_str);
    }
}