<?php

class GTCSyncClient extends GTCBase {

    private $subdomain;
    private $username;
    private $password;

    public function __construct($subdomain, $username, $password){
        $this->subdomain = $subdomain;
        $this->username = $username;
        $this->password = $password;
        $this->token = $this->getToken();
    }

    private function makeRequest($url, $data, $params = null, $method = 'POST'){

        $is_login = $url == 'api/login';

        if (empty($this->token) && !$is_login)
            $this->login();

        // Build URL. If The IP is specified, make requests without
        // domain name.
        $ip = GTCSYNC_IP;
        $absolute_url = empty($ip) ? GTCSYNC_URL : GTCSYNC_IP;
        $absolute_url .= '/'.$url;
        if ($params != null)
            $absolute_url.= '?'.http_build_query($params);

        // Set headers.
        // Explicitly add the host header as we may be making the request
        // directly to the server IP
        $domain = parse_url(GTCSYNC_URL)['host'];
        $headers = [
            'User-Agent' => GTCSYNC_ID,
            'Host' => $domain,
        ];
        if ($this->token)
            $headers['Authorization'] = 'Bearer '.$this->token;

        // Add some diagnostic data to login requests
        if (GTCSYNC_ANALYTICS && $is_login){
            try {
                $system_info = $this->getSystemInfo();
                $headers['GTC-Site-URL'] = get_site_url();
                $headers['GTC-Plugin-Version'] = GTCSYNC_VERSION;
                $headers['GTC-PHP-Version'] = $system_info['php_version'];
                $headers['GTC-WP-Version'] = $system_info['wordpress_version'];
                $headers['GTC-WC-Version'] = $system_info['woocommerce_version'];

                if ($this->acceptedEula()) {
                    $options = [
                        'global_attributes' => boolval(get_option(GTCSYNC_PREFIX . 'global_attributes')),
                        'import_product_batch_count' => intval(get_option(GTCSYNC_PREFIX . 'import_product_batch_count')),
                        'sync_enabled' => boolval(get_option(GTCSYNC_PREFIX . 'sync_enabled')),
                        'sync_adjustment_enabled' => boolval(get_option(GTCSYNC_PREFIX . 'sync_adjustment_enabled')),
                        'sync_cache_seconds' => intval(get_option(GTCSYNC_PREFIX . 'sync_cache_seconds')),
                        'background_sync_enabled' => boolval(get_option(GTCSYNC_PREFIX . 'background_sync_enabled')),
                        'background_sync_interval_seconds' => intval(get_option(GTCSYNC_PREFIX . 'background_sync_interval_seconds')),
                        'background_sync_last_synced' => get_option(GTCSYNC_PREFIX . 'background_sync_last_synced'),
                        'background_sync_flush_cache' => get_option(GTCSYNC_PREFIX . 'background_sync_flush_cache'),
                    ];

                    $product_counts = [
                        'any' => $this->getProductCount(),
                        'publish' => $this->getProductCount('publish'),
                        'draft' => $this->getProductCount('draft'),
                    ];

                    $extra = [
                        'cron' => $system_info['cron'],
                        'timezone' => $system_info['timezone'],
                        'product_counts' => $product_counts,
                        'options' => $options,
                        'plugins' => $this->getActivePlugins(),
                        'site_url' => get_site_url(),
                    ];
                    $headers['GTC-Plugin-Extra'] = base64_encode(json_encode($extra));
                }
            } catch (\Exception $e) {
                $e = new \Exception('Failed to add analytics headers.', 500, $e);
                $this->logError($e, $headers, 'warning');
            }
        }

        $response = wp_remote_post(
            $absolute_url,
            [
                'method' => $method,
                'timeout' => 45,
                'redirection' => 5,
                'httpversion' => '1.1',
                'blocking' => true,
                'headers' => $headers,
                'body' => $data,
                'cookies' => [],
                // Disable cert validation when using IP, as the IP will not match the cert's domain
                'sslverify' => empty($ip),
            ]
        );

        // Check for transport error
        $this->handleWpError($response);

        $body = $response['body'];
        $response_data = json_decode($body, true);
        $response_ex = new Exception('HTTP Body - '.$body, $response['response']['code']);

        // json_decode returns null on parse error
        if ($response_data === null)
            throw $response_ex;

        if (array_key_exists('message', $response_data)){

            $message = $response_data['message'];

            // Invalid credentials
            if ($message == 'invalid_credentials')
                throw new Exception('Login details incorrect', 0, $response_ex);

            // If the token has expired, login again and resend request
            if (in_array($message, ['token_expired', 'Token has expired', 'Unauthorized'])) {
                $this->login();
                return $this->makeRequest($url, $data, $params, $method);
            }

            // Check for not okay response
            $message_whitelist = ['SKU not found!'];
            if ($response['response']['code'] != 200 &&
                empty($message) == false &&
                in_array($message, $message_whitelist) == false) {

                throw new Exception($message, 0, $response_ex);
            }
        }

        if (array_key_exists('error', $response_data)){
            $error = $response_data['error'];
            throw new Exception($error, 0, $response_ex);
        }

        return $response_data;
    }

    private function getToken(){
        return get_option(GTCSYNC_PREFIX.'token');
    }

    private function setToken($token){
        $this->token = $token;
        update_option(GTCSYNC_PREFIX.'token', $token);
    }

    public function hasCredentials(){
        return empty($this->subdomain) == false &&
            empty($this->username) == false &&
            empty($this->password) == false;
    }

    public function login(){

        $this->setToken('');
        update_option(GTCSYNC_PREFIX.'outlet_id', null);
        update_option(GTCSYNC_PREFIX.'outlet_name', null);

        try {
            $data = $this->makeRequest(
                'api/login',
                [
                    'subdomain' => $this->subdomain,
                    'username' => $this->username,
                    'password' => $this->password,
                ]
            );
        } catch (Throwable $e) {
            $e = new \Exception('Failed to login - '.$e->getMessage(), 401, $e);
            $this->logError($e, [
                'subdomain' => $this->subdomain,
                'username' => $this->username,
            ]);
            throw $e;
        }

        // Check for operator user
        if ($data['user_level'] == 'outlet_operator')
            throw new \Exception('Operator accounts cannot be used. Please use a store owner or admin account instead.');

        // Save details
        update_option(GTCSYNC_PREFIX.'outlet_id', $data['current_outlet_id']);
        update_option(GTCSYNC_PREFIX.'outlet_name', $data['current_outlet_name']);

        $this->setToken($data['token']);

        return $data;
    }

    public function getProducts(){

        $data = $this->makeRequest('api/ecommerce/products', [], null, 'GET');
        return $data;
    }

    public function getStockCount($sku){

        $params = ['sku' => $sku];
        $data = $this->makeRequest('api/ecommerce/get_inventory', [], $params, 'GET');

        // Check for not found
        if ($data['status'] === false)
            return null;

        // Don't return stock count if there is more than one match
        if (count($data['data']) != 1)
            return null;

        return $data['data'][0];
    }

    public function getAllStockCounts(){

        $data = $this->makeRequest('api/ecommerce/get_inventory', [], null, 'GET');
        return $data['data'];
    }

    public function adjustStockCount($products){

        $form = ['products' => $products];
        return $this->makeRequest('api/ecommerce/adjust_inventory', $form, null, 'POST');
    }

    public function getTaxRates(){

        $data = $this->makeRequest('api/ecommerce/vat_rates', [], null, 'GET');
        $data = $data['data'];

        usort($data, function($a, $b){
            return $a['vat_rate'] < $b['vat_rate'];
        });

        return $data;
    }
}