<?php

class GTCSyncCore extends GTCBase {

    CONST STOCK_UPDATE_KEY = 'gtc-stock-updated-date';
    CONST STOCK_UPDATE_FAILED_KEY = '_gtc-stock-update-failed-date';
    CONST STOCK_IMPORT_KEY = 'gtc-import-date';
    CONST STOCK_TAKEN_KEY = '_gtc-stock-taken';
    CONST STOCK_VERSION_KEY = '_gtc-stock-version';
    CONST STOCK_UPDATE_REASON_KEY = '_gtc-stock-update-reason';

    CONST STOCK_UPDATE_REASON_BACKGROUND = 'background_sync';
    CONST STOCK_UPDATE_REASON_VIEW = 'product_view';

    public function __construct() {

        // Check if plugin is configured before adding hooks and filters
        if ($this->isConfigured() == false)
            return;

        // Pull stock from GTC
        add_filter('woocommerce_product_is_in_stock', [$this, 'updateProductStock'], 0, 2);

        // Update stock in GTC
        add_action('woocommerce_order_status_pending', [$this, 'handleStatusPending'], 20);
        add_action('woocommerce_order_status_processing', [$this, 'handleStatusProcessing'], 20);
        add_action('woocommerce_order_status_on-hold', [$this, 'handleStatusOnHold'], 20);
        add_action('woocommerce_order_status_completed', [$this, 'handleStatusCompleted'], 20);
        add_action('woocommerce_order_status_cancelled', [$this, 'handleStatusCancelled'], 20);
        add_action('woocommerce_order_status_failed', [$this, 'handleStatusFailed'], 20);

        add_action('woocommerce_restock_refunded_item', [$this, 'restockRefundedItem'], 10, 5 );

        // Product edit panel
        add_filter('woocommerce_product_data_tabs', [$this, 'productDataTabs']);
        add_action('woocommerce_product_data_panels', [$this, 'productDataContent']);
    }

    public function updateProductStock($in_stock, WC_Product $product){

        // Ignore if disabled
        if (get_option(GTCSYNC_PREFIX.'sync_enabled') == false)
            return $in_stock;

        // This filter fires:
        // - Regardless of whether the product is in stock
        // - Regardless of whether inventory management is enabled
        // - On any pages where the product "in stock"/"out of stock" text is displayed.
        // - Before wc begins to write the product details to the page
        // Can fire multiple times but the cache check will prevent us doing
        // the work multiple times.

        // Only update if the user is viewing a product - do not update in admin,
        // as there may be tables containing many products causing slowdown.
        if (is_product() == false || is_admin())
            return $in_stock;

        // Check cache timeout
        if (static::stockCacheExpired($product) === false)
            return $in_stock;

        // Use passed product, as this may differ to the post product_id,
        // especially in the case of variable products.
        $sku = $product->get_sku();

        $debug = [
            'wc_product_sku' => $sku,
            'wc_product_id' => $product->get_id(),
            'gtc_product_id' => $product->get_meta('gtc-product-id'),
            'gtc_import_version' => $product->get_meta('gtc-import-version'),
            'gtc_import_date' => $product->get_meta(static::STOCK_IMPORT_KEY),
            'gtc_stock_updated_date' => $product->get_meta(static::STOCK_UPDATE_KEY),
            'gtc_stock_update_failed_date' => $product->get_meta(static::STOCK_UPDATE_FAILED_KEY),
            'gtc_stock_update_reason' => $product->get_meta(static::STOCK_UPDATE_REASON_KEY),
            'sync_cache_seconds' => get_option(GTCSYNC_PREFIX.'sync_cache_seconds'),
        ];

        try {
            // If no SKU, don't try to sync
            if (empty($sku))
                return $in_stock;

            $inventory_info = $this->getClient()->getStockCount($sku);

            $debug['inventory_info'] = $inventory_info;

            // Update reason for updating the stock
            $key = static::STOCK_UPDATE_REASON_KEY;
            $product->update_meta_data($key, static::STOCK_UPDATE_REASON_VIEW);

            if ($inventory_info === null){
                // One or more products for this SKU,
                // return existing stock count
                $this->logError(new Exception('Could not get stock level for SKU'), $debug, 'warning');

                // Record stock update failure
                $value = (new DateTime)->format(GTCSYNC_DATEFORMAT);
                $product->update_meta_data(static::STOCK_UPDATE_FAILED_KEY, $value);
                $product->save();
            } else {

                // Change semantics of values in to woo format
                $data = $this->convertInventoryData(
                    $inventory_info['track_inventory'],
                    $inventory_info['inventory']
                );
                $track_inventory = $data['track_inventory'];
                $count = $data['quantity'];

                // Update in_stock value
                if ($track_inventory)
                    $in_stock = $count > 0;

                // Update the product stock quantity and managed flag
                $product->set_manage_stock($track_inventory);
                $product->set_stock_quantity($count);

                // Update stock last updated time.
                $value = (new DateTime)->format(GTCSYNC_DATEFORMAT);
                $product->update_meta_data(static::STOCK_UPDATE_KEY, $value);

                $product->save();
            }
        } catch (Throwable $e){
            $e = new Exception('Could not update stock level for SKU', 500, $e);
            $this->logError($e, $debug);

            // Record stock update failure
            $value = (new DateTime)->format(GTCSYNC_DATEFORMAT);
            $product->update_meta_data(static::STOCK_UPDATE_FAILED_KEY, $value);
            $product->save();
        }

        // Return the new in_stock value
        return $in_stock;
    }

    public function handleStatusPending($order_id){

        $this->handleState($order_id, 'pending');
    }

    public function handleStatusProcessing($order_id){

        $this->handleState($order_id, 'processing');
    }

    public function handleStatusOnHold($order_id){

        $this->handleState($order_id, 'on-hold');
    }

    public function handleStatusCompleted($order_id){

        $this->handleState($order_id, 'completed');
    }

    public function handleStatusCancelled($order_id){

        $this->handleState($order_id, 'cancelled');
    }

    public function handleStatusRefunded($order_id){

        $this->handleState($order_id, 'refunded');
    }

    public function handleStatusFailed($order_id){

        $this->handleState($order_id, 'failed');
    }

    public function handleState($order_id, $state){

        // Ignore if disabled
        if (get_option(GTCSYNC_PREFIX.'sync_adjustment_enabled') == false)
            return;

        $order = new WC_Order($order_id);

        // Store whether we need to take stock from inventory for each state
        $states_stock = [
            'pending' => true,
            'processing' => true,
            'on-hold' => true,
            'completed' => true,
            'cancelled' => false,
            'failed' => false,
        ];

        // Get whether the stock is currently taken
        if ($order->meta_exists(static::STOCK_TAKEN_KEY) == false)
            $order->add_meta_data(static::STOCK_TAKEN_KEY, 'false');
        $stock_taken = $order->get_meta(static::STOCK_TAKEN_KEY) === 'true';

        // Get whether stock should be taken
        if (isset($states_stock[$state]) == false){
            $this->logError(new Exception('Unknown state '.$state, 500));
            return;
        }

        $stock_should_be_taken = $states_stock[$state];
        $updated = false;

        // We've taken stock and need to return it (cancellation, refund)
        if ($stock_taken && $stock_should_be_taken == false) {
            $this->incrementOrderProductStock($order);
            $order->update_meta_data(static::STOCK_TAKEN_KEY, 'false');
            $updated = true;
        }
        // We need to take stock (pending)
        else if ($stock_taken == false && $stock_should_be_taken) {
            $this->decrementOrderProductStock($order);
            $order->update_meta_data(static::STOCK_TAKEN_KEY, 'true');
            $updated = true;
        }

        // Record the version used to update the store
        if ($updated && $order->meta_exists(static::STOCK_VERSION_KEY) == false)
            $order->add_meta_data(static::STOCK_VERSION_KEY, GTCSYNC_VERSION);

        $order->save();
    }

    public function restockRefundedItem($product_id, $old_stock, $new_stock, WC_Order $order, WC_Product $product){

        $debug = ['wc_order_id' => $order->get_id(), 'wc_product_id' => $product->get_id()];

        try {
            $sku = $product->get_sku();

            // Can't sync stock for empty SKU, so cancel
            if (empty($sku))
                return;

            $quantity = $new_stock - $old_stock;

            $request = [['sku' => $sku, 'quantity' => $quantity * -1]];
            $debug['api_request'] = $request;
            $response = $this->getClient()->adjustStockCount($request);

            $message = sprintf(
                "Item restocked in Goodtill due to individual item refund -\n#%s %s x %s.",
                $product_id,
                $sku,
                $quantity
            );
            $order->add_order_note($message, 0, false);
        } catch (\Exception $e) {
            $e = new Exception('Cannot update single item stock', 500, $e);
            $this->logError($e, $debug);
        }
    }

    private function adjustOrderProductStock(WC_Order $order, $positive){

        $order_id = $order->get_id();

        // Use positive quantities for decrementing, negative quantities
        // for incrementing
        $multiplier = $positive ? -1 : 1;

        $debug = ['wc_order_id' => $order_id, 'multiplier' => $multiplier];

        try {
            $request = [];
            $sku_string = '';

            foreach ($order->get_items() as $item){

                // Send quantity for products regardless of whether stock is
                // managed or not. GTC backend will ignore items that are not
                // managed at it's end, and the wc value for managed may not
                // be up to date.

                /** @var WC_Product $product */
                $product = $item->get_product();

                $sku = $product->get_sku();
                $product_id = $product->get_id();

                // Can't sync stock for empty SKU, so skip
                if (empty($sku))
                    continue;

                $quantity = $item->get_quantity();

                $request[] = ['sku' => $sku, 'quantity' => $quantity * $multiplier];
                $sku_string .= sprintf("\n#%s %s x %s", $product_id, $sku, $quantity);
            }

            // Possible to have an order without items, ignore in this case
            if (empty($request))
                return $order_id;

            $debug['api_request'] = $request;

            try {
                $response = $this->getClient()->adjustStockCount($request);

                $message = sprintf(
                    "All order items %s in Goodtill due to order status change -%s",
                    $positive ? 'restocked' : 'stock reduced',
                    $sku_string
                );
                $order->add_order_note($message, 0, false);
            } catch (Throwable $e){
                $e = new Exception('Cannot update order stock', 500, $e);
                $this->logError($e, $debug);
            }
        } catch (Throwable $e){
            $e = new Exception('Cannot update order stock', 500, $e);
            $this->logError($e, $debug);
        }

        return $order_id;
    }

    public function decrementOrderProductStock($order){

        return $this->adjustOrderProductStock($order, false);
    }

    public function incrementOrderProductStock($order){

        return $this->adjustOrderProductStock($order, true);
    }

    public static function stockCacheExpired(WC_Product $product){

        $success_expired = static::stockCacheExpiredForKey($product, static::STOCK_UPDATE_KEY);
        $failed_expired = static::stockCacheExpiredForKey($product, static::STOCK_UPDATE_FAILED_KEY);

        return $success_expired && $failed_expired;
    }

    private static function stockCacheExpiredForKey(WC_Product $product, $key){

        // No last updated time
        if ($product->meta_exists($key) == false)
            return true;

        $stock_updated_string = $product->get_meta($key);
        $stock_updated_date = DateTime::createFromFormat(GTCSYNC_DATEFORMAT, $stock_updated_string);

        // Can't parse last updated time, assume invalid
        if ($stock_updated_date === false)
            return true;

        $stock_updated_seconds_ago = (new DateTime)->getTimestamp() - $stock_updated_date->getTimestamp();

        $stock_cache_sessions = get_option(GTCSYNC_PREFIX . 'sync_cache_seconds');

        // Can't get cache time, assume invalid
        if (empty($stock_cache_sessions))
            return true;

        return $stock_updated_seconds_ago >= $stock_cache_sessions;
    }

    function productDataTabs($tabs){

        $tabs['gtc'] = [
            'label'     => 'Goodtill',
            'target'    => 'gtc_options',
            'class'     => ''
        ];
        return $tabs;
    }

    function productDataContent(){

        // Get product
        global $post;
        $product = wc_get_product($post->ID);

        $is_variable = $product->get_type() == 'variable';

        $stock_info = $this->getStockUpdateInfo($product);

        if ($is_variable == false) {
            $stock_update_lines = [
                [
                    'sku' => $product->get_sku(),
                    'text' => $stock_info['text'],
                    'stock_update_success' => $stock_info['last_update_successful'],
                    'manage_stock' => $product->get_manage_stock(),
                    'stock_quantity' => $product->get_stock_quantity(),
                ]
            ];
        }

        foreach ($product->get_children() as $child_id){
            $variant = wc_get_product($child_id);
            $stock_info = $this->getStockUpdateInfo($variant);
            $stock_update_lines[] = [
                'sku' => $variant->get_sku(),
                'text' => $stock_info['text'],
                'stock_update_success' => $stock_info['last_update_successful'],
                'manage_stock' => $variant->get_manage_stock(),
                'stock_quantity' => $variant->get_stock_quantity(),
            ];
        }

        $import_date = $product->get_meta(static::STOCK_IMPORT_KEY);

        $last_sync = get_option(GTCSYNC_PREFIX.'background_sync_last_synced');
        if (empty($last_sync) == false)
            $last_sync = $this->setTimeZone($last_sync);

        // Note the 'id' attribute needs to match the 'target' parameter set above
        ?><div id='gtc_options' class='panel woocommerce_options_panel'>
            <?php if ($product->meta_exists('gtc-product-id')) { ?>
                <div class='options_group'>
                    <p class="form-field">
                        <label>View on Goodtill</label>
                        <a href="<?php echo GTCSYNC_APP; ?>/app/products/edit/<?php echo $product->get_meta('gtc-product-id'); ?>"
                            target="_blank">
                            View / Edit
                        </a>
                        <?php if ($is_variable) { ?>
                            | <a href="<?php echo GTCSYNC_APP; ?>/app/products/view/<?php echo $product->get_meta('gtc-product-id'); ?>"
                            target="_blank">Variants</a>
                        <?php } ?>
                    </p>
                    <p class="form-field">
                        <label>Import date</label>
                        <span><?php echo $import_date; ?></span>
                    </p>
                </div>
            <?php } else { ?>
                <p>Product has not been imported from Goodtill.</p>
            <?php } ?>
            <div class='options_group' style="padding: 10px;">
                <table class="widefat">
                    <thead>
                        <tr>
                            <th>SKU</th>
                            <th>Stock managed</th>
                            <th>Stock quantity</th>
                            <th>Stock sync status</th>
                        </tr>
                    </thead>
                    <tr>
                        <?php foreach ($stock_update_lines as $item) { ?>
                            <tr>
                                <td><?php echo $this->skuHtml($item['sku']); ?></td>
                                <td><?php echo $item['manage_stock'] ? 'Yes' : 'No'; ?></td>
                                <td><?php echo $item['stock_quantity']; ?></td>
                                <td><?php printf('<span style="color: %s">%s</span>', $item['stock_update_success'] ? 'green' : 'red', $item['text']); ?></td>
                            <tr>
                        <?php } ?>
                    </tr>
                </table>
                <span class="wrap">
                    <br>If any of the above SKUs show 'Failed' please check that the SKUs are not empty, unique
                    and match in both WooCommerce and Goodtill.
                    <?php if ($this->hasCachePlugin()) { ?>
                        <br><br><strong>It looks like you may be be using a cache plugin - please note these may affect
                        stock syncing interval and/or cause an outdated stock quantity to be displayed to your
                        users.</strong>
                    <?php } ?>
                    <?php if ($product->get_status() != 'publish') { ?>
                        <br><br><strong>Stock for this product will not be updated in the next background because the status
                        is not "published".</strong>
                    <?php } ?>
                </span>
            </div>
            <div class="options_group">
                <p class="form-field">
                    <label>Last background sync</label>
                    <span class="wrap">
                        <?php echo empty($last_sync) ? 'Never synced' : $last_sync; ?>
                    </span>
                </p>
            </div>
        </div><?php
    }
}

$GLOBALS['gtcsync_core'] = new GTCSyncCore();