<?php

class GTCSyncCron extends GTCBase {

    const HOOK_NAME = 'gtc_cron';
    private static $loaded = false;
    private $schedule_key = 'gtc_schedule';
    private $all_inventory;
    private $data_key = GTCSYNC_PREFIX.'cron_temp';

    function __construct(){

        if (static::$loaded == false)
            add_action('wp_loaded', [$this, 'init']);
    }

    public function init(){

        static::$loaded = true;

        // Stop if disabled
        if (get_option(GTCSYNC_PREFIX.'background_sync_enabled') == false)
            return;

        // Stop if not configured
        if ($this->isConfigured() == false)
            return;

        $schedule_interval_seconds = get_option(GTCSYNC_PREFIX.'background_sync_interval_seconds');
        add_filter('cron_schedules', function($schedules) use ($schedule_interval_seconds) {
            $minutes = $schedule_interval_seconds >= 60;
            $schedules[$this->schedule_key] = [
                'interval' => $schedule_interval_seconds,
                'display'  => sprintf(
                    '%s %s',
                    $minutes ? intval($schedule_interval_seconds / 60) : $schedule_interval_seconds,
                    $minutes ? 'minutes' : 'seconds'
                ),
            ];
            return $schedules;
        });

        add_action(static::HOOK_NAME, [$this, 'runUpdate']);

        if (!wp_next_scheduled(static::HOOK_NAME))
            wp_schedule_event(time(), $this->schedule_key, static::HOOK_NAME);
    }

    public function runUpdate(){

        try {
            // Update last synced time
            $now = (new DateTime())->format(GTCSYNC_DATEFORMAT);
            update_option(GTCSYNC_PREFIX.'background_sync_last_synced', $now);

            // Get WC product IDs. Use leftover IDs or get a new list.
            $product_ids = null;
            $index = 0;
            if ($this->isInProgress()) {
                $data = $this->getData();
                $index = $data['index'];
                $product_ids = $data['product_ids'];
                $this->getLogger()->info('Resumed cron update at index ' . $index);
            } else {
                $product_ids = $this->getProductIds();
                $this->updateData(true, $index, $product_ids);
                $this->getLogger()->info('Started cron update of ' . count($product_ids) . ' products');
            }

            // Download all product inventory
            $all_stock_counts = $this->getClient()->getAllStockCounts();
            $this->all_inventory = [];
            foreach ($all_stock_counts as $item) {
                $this->all_inventory[$item['product_sku']] = $this->convertInventoryData(
                    $item['track_inventory'],
                    $item['inventory']
                );
            }
            $all_stock_counts = null;

            $this->start = microtime(true);

            // Loop through products
            while ($index < count($product_ids)) {
                // Update product
                $product_id = $product_ids[$index];
                $this->updateProduct($product_id);
                // Save next index
                $index++;
                $this->updateData(true, $index);
            }

            $this->updateData(false, 0);

            // We're done. Purge the cache if requested.
            if (get_option(GTCSYNC_PREFIX.'background_sync_flush_cache') == 'all') {
                $this->clearAllCache();
                $this->getLogger()->info('Flushed all cached pages');
            }

        } catch (Throwable $e) {
            $e = new Exception('Failed to run cron update', 500, $e);
            $this->logError($e);
            throw $e;
        }
    }

    private function updateProduct($product_id){

        // Get product
        $wc_product = wc_get_product($product_id);

        // Product may not exist somehow, skip in this case
        if ($wc_product == null) {
            $e = new Exception('Product for background stock update does not exist.', 500);
            $this->logError($e, ['product_id' => $product_id]);
            return false;
        }

        // If variable, update children
        foreach ($wc_product->get_children() as $child_id)
            $this->updateProduct($child_id);

        // Update reason for updating the stock
        $key = GTCSyncCore::STOCK_UPDATE_REASON_KEY;
        $this->createOrUpdateMeta($wc_product, $key, GTCSyncCore::STOCK_UPDATE_REASON_BACKGROUND);

        // Check SKU
        $sku = $wc_product->get_sku();

        $can_update = empty($sku) == false && isset($this->all_inventory[$sku]);
        if ($can_update){
            // Get from WC and perform update
            $inventory = $this->all_inventory[$sku];

            $wc_product->set_manage_stock($inventory['track_inventory']);
            $wc_product->set_stock_quantity($inventory['quantity']);

            // Update stock last updated time.
            $key = GTCSyncCore::STOCK_UPDATE_KEY;
            $value = (new DateTime)->format(GTCSYNC_DATEFORMAT);
            $this->createOrUpdateMeta($wc_product, $key, $value);

            // Purge product page cache, if requested
            if (get_option(GTCSYNC_PREFIX.'background_sync_flush_cache') == 'product_posts')
                $this->clearProductCache($wc_product->get_id());
        } else {
            // Update stock last updated time.
            $key = GTCSyncCore::STOCK_UPDATE_FAILED_KEY;
            $value = (new DateTime)->format(GTCSYNC_DATEFORMAT);
            $this->createOrUpdateMeta($wc_product, $key, $value);
        }

        $wc_product->save();

        return $can_update;
    }

    private function createOrUpdateMeta(WC_Product $wc_product, $key, $value){
        if ($wc_product->meta_exists($key))
            $wc_product->update_meta_data($key, $value);
        else
            $wc_product->add_meta_data($key, $value);
    }

    private function getProductIds() {

        // Get all published product IDs.
        // This does not provide variant IDs,
        $params = [
            'posts_per_page' => -1,
            'type' => ['simple', 'variable'],
            'status' => 'publish',
            'return' => 'ids',
        ];

        return wc_get_products($params);
    }

    public function isInProgress(){

        $data = $this->getData();

        if ($data == null)
            return false;

        return $data['in_progress'];
    }

    public function getData(){

        return get_option($this->data_key);
    }

    private function updateData($in_progress, $index, $product_ids = null){

        $option_data = $this->getData();

        $option_data['in_progress'] = $in_progress;
        $option_data['index'] = $index;
        $option_data['updated'] = (new DateTime)->format(GTCSYNC_DATEFORMAT);

        if ($product_ids) {
            $option_data['product_ids'] = $product_ids;
            $option_data['started'] = (new DateTime)->format(GTCSYNC_DATEFORMAT);
        }

        update_option($this->data_key, $option_data, false);
    }

    public static function unregister() {
        $timestamp = wp_next_scheduled(static::HOOK_NAME);
        wp_unschedule_event($timestamp, static::HOOK_NAME);
    }

    public static function nextSync() {
        $timestamp = wp_next_scheduled(static::HOOK_NAME);

        if ($timestamp == false)
            return 'Not scheduled';

        return (new DateTime('Europe/London'))
            ->setTimestamp($timestamp)
            ->format(GTCSYNC_DATEFORMAT);
    }
}

$GLOBALS['gtcsync_cron'] = new GTCSyncCron();