<?php

class GTCSyncExport extends GTCBase {

    private $tax_rates = null;
    private $data_key = GTCSYNC_PREFIX.'export_temp';
    private $settings_id = GTCSYNC_ID.'-settings';
    private $products_per_page = 10;

    function __construct(){

        add_action('admin_init', function() {
            register_setting($this->settings_id.'_export', $this->data_key);
        });
    }

    private function getProducts($page, $type, $created_date_from, $created_date_to){

        $data = [
            'orderby' => 'id',
            'order' => 'desc',
            'per_page' => $this->products_per_page,
            'page' => $page,
            'paged'=> '1',
            'status' => $type,
        ];

        if (empty($created_date_from) == null)
            $data['after'] = $created_date_from;

        if (empty($created_date_to) == null)
            $data['before'] = $created_date_to;

        $wp_rest_request = new WP_REST_Request('GET');
        $wp_rest_request->set_query_params($data);
        $products_controller = new WC_REST_Products_Controller;
        $products_response = @$products_controller->get_items($wp_rest_request);
        $this->handleWpError($products_response);
        return $products_response->get_data();
    }

    private function getVariations($product, $page){

        $data = [
            'orderby' => 'id',
            'paged' => '1',
            'per_page' => 3,
            'page' => $page,
            'status' => 'publish',
        ];
        $wp_rest_request = new WP_REST_Request('GET');
        $wp_rest_request->set_query_params($data);
        $wp_rest_request['product_id'] = $product['id'];
        $variations_controller = new WC_REST_Product_Variations_Controller;
        $variations_response = @$variations_controller->get_items($wp_rest_request);
        $this->handleWpError($variations_response);
        return $variations_response->get_data();
    }

    private function getHeader(){

        return [
            'product_name',
            'category',
            'has_variant',
            'has_attributes',
            'display_attr_pos',
            'is_child_product',
            'display_order',
            'display_name',
            'selling_price',
            'vat_code',
            'shareable',
            'track_inventory',
            'inventory',
            'product_tags',
            'brand',
            'barcode',
            'weight',
            'product_sku',
            'attribute_1_name',
            'attribute_1_value',
            'attribute_2_name',
            'attribute_2_value',
            'attribute_3_name',
            'attribute_3_value',
            'product_desc',
            'category_desc',
            'image',
            'wc_product_id',
            'wc_created_date',
            'wc_gtc_stock_updated_date',
            'imported_from_gtc',
            'gtc_product_id',
        ];
    }

    private function productToRow($product, $parent_product=null, $images=true)
    {
        $export_date_format = 'Y-m-d H:i:s';

        $is_variant = $parent_product != null;
        $parent_or_child = $is_variant ? $parent_product : $product;

        // Prepare key => value array of attributes.
        // - Remove attributes not used for variants.
        // - Ensure that all parent attributes are present for children.
        $attrs = [];
        if ($is_variant == false){
            foreach ($product['attributes'] as $attr){
                // Ignore attributes not used for variants
                if ($attr['variation'] == false)
                    continue;

                $attrs[$attr['name']] = '';
            }
        }
        else {

            foreach ($parent_product['attributes'] as $parent_attr){

                // Ignore attributes not used for variants
                if ($parent_attr['variation'] == false)
                    continue;

                $name = $parent_attr['name'];
                $value = '';

                // Get value from child
                foreach ($product['attributes'] as $child_attr){
                    if ($child_attr['name'] == $name)
                        $value = $child_attr['option'];
                }

                $attrs[$name] = $value;
            }
        }

        // Tags
        $tags = '';
        if (isset($parent_or_child['tags'])){
            $tags = array_map(function($tag){
                return html_entity_decode($tag['name']);
            }, $parent_or_child['tags']);
            $tags = implode('#', $tags);
        }

        // Category
        // Either take the primary category set by Yoast SEO, or take the first item.
        // Ignore the uncategoriesd categories.
        $category = '';
        $category_desc = '';
        $categories = array_values(array_filter($parent_or_child['categories'], function($category){
            return in_array(strtolower($category['name']), ['uncategorized', 'uncategorised']) == false;
        }));
        $primary_category_id = $this->getMeta($parent_or_child, '_yoast_wpseo_primary_product_cat');
        if (empty($categories) == false){

            // Use first as default
            $category_item = $categories[0];

            // Check for Yoast primary category
            if ($primary_category_id != null){
                foreach ($categories as $search_category){
                    if ($search_category['id'] == $primary_category_id){
                        $category_item = $search_category;
                        break;
                    }
                }
            }

            // Get details for selected category. Get the term from WordPress as the array we get from WC
            // does not include the description.
            $cat_term = get_term($category_item['id'], 'product_cat');

            $category = html_entity_decode($cat_term->name);
            $category_desc = $this->htmlToPlainText($cat_term->description);
        }

        // Brand
        // Fetched from 'brand' when when using the WC brand plugin - https://woocommerce.com/products/brands
        // Either take the primary brand set by Yoast SEO, or take the first item.
        $brand = '';
        if (isset($parent_or_child['brands']) && empty($parent_or_child['brands']) == false){

            // Use first as default
            $brand_item = $parent_or_child['brands'][0];

            // Check for Yoast primary category
            $primary_brand_id = $this->getMeta($parent_or_child, '_yoast_wpseo_primary_product_brand');
            if ($primary_brand_id != null){
                foreach ($parent_or_child['brands'] as $search_brand){
                    if ($search_brand['id'] == $primary_brand_id){
                        $brand_item = $search_brand;
                        break;
                    }
                }
            }

            $brand = html_entity_decode($brand_item['name']);
        }
        // Otherwise look for an attribute called 'brand' (case insensitive).
        if (empty($brand)) {
            foreach ($parent_or_child['attributes'] as $attribute){
                if (isset($attribute['options'][0]) && strtolower($attribute['name']) == 'brand') {
                    $brand = html_entity_decode($attribute['options'][0]);
                    break;
                }
            }
        }

        // Get name
        // For parent products, use the full name
        // For variants, use the attribute values
        $name = '';
        // Only products have a name
        if (isset($product['name'])) {
            $name = html_entity_decode($product['name']);
            $display_name = $name;
        }
        // Create the variant name from attribute values
        else if ($is_variant){

            // Get a list of all attribute values for this variation
            // that require a specific option to be
            $attribute_options = array_values(array_filter($attrs));

            $display_name = implode(', ', $attribute_options);

            $name = html_entity_decode($parent_product['name']);
            if (empty($attribute_options) == false)
                $name .= ' - '.$display_name;
        }

        // Get price. In GTC all products need prices, even variant parent
        // products, but this is not the case in wc. Use the regular_price
        // and if that is empty, in the case of a variable product, get the
        // calculated 'price' attribute. It should not be possible for both
        // price attributes to be empty, as we are filtering "purchasable"
        // products before this point.
        $price = $product['regular_price'];
        if (empty($price))
            $price = $product['price'];

        // Product description (convert to plain text)
        $description = $this->htmlToPlainText($product['description']);

        // Get base64 version of featured image
        $image_base64 = '';
        if ($images){

            // Get the URL from Wordpress instead of the collection from the
            // API. API URLs are full resolution and may cause us to run out
            // of memory when resizing, 'medium' size post thumbnail is 300px
            // cropping to aspect ratio.
            $url = get_the_post_thumbnail_url($product['id'], 'medium');

            if ($url){
                // API returns a URL, which we could use, but we may be
                // blocked by firewalls. Instead, convert this in to
                // a path to the file on disk.
                $url_info = parse_url($url);
                $url = $this->joinPaths(get_home_path(), $url_info['path']);

                // Get settings
                $settings = explode(',', GTCSYNC_IMAGESETTINGS);
                $size = intval($settings[0]);
                $quality = intval($settings[1]);

                $image_base64 = $this->getThumbnail($url, $size, $quality);
            }
        }

        // Get meta fields
        $gtc_import_date = '';
        $gtc_stock_updated_date = '';
        $gtc_product_id = '';
        foreach ($product['meta_data'] as $meta){
            if ($meta->key == 'gtc-import-date')
                $gtc_import_date = DateTime::createFromFormat(GTCSYNC_DATEFORMAT, $meta->value)->format($export_date_format);
            else if ($meta->key == 'gtc-stock-updated-date')
                $gtc_stock_updated_date = DateTime::createFromFormat(GTCSYNC_DATEFORMAT, $meta->value)->format($export_date_format);
            else if ($meta->key == 'gtc-product-id')
                $gtc_product_id = $meta->value;
        }

        $new_row = [];
        $new_row['product_name'] = $name;
        $new_row['category'] = $category;
        $new_row['has_variant'] = $is_variant == false && $product['type'] == 'variable' ? '1' : '';
        $new_row['has_attributes'] = $new_row['has_variant'];
        $new_row['display_attr_pos'] = $new_row['has_variant'];
        $new_row['is_child_product'] = $is_variant ? '1' : '';
        $new_row['display_order'] = $product['menu_order'];
        $new_row['display_name'] = $display_name;
        $new_row['selling_price'] = $price;
        $new_row['vat_code'] = $this->getTaxRate($product);
        $new_row['shareable'] = '1';
        $new_row['track_inventory'] = $product['manage_stock'] ? '1' : '';
        $new_row['inventory'] = $product['stock_quantity'];
        $new_row['product_tags'] = $tags;
        $new_row['brand'] = $brand;
        $new_row['barcode'] = '';
        $new_row['weight'] = $this->getWeight($product);
        $new_row['product_sku'] = $product['sku'];

        // Add placeholders for up to 3 attributes
        for ($index = 1; $index <= 3; $index++){
            $new_row['attribute_'.$index.'_name'] = '';
            $new_row['attribute_'.$index.'_value'] = '';
        }

        // Add up to 3 attributes
        $product_attributes = array_slice($attrs, 0, 3);
        $position = 1;
        foreach ($product_attributes as $name => $value){
            $new_row['attribute_'.$position.'_name'] = $name;
            $new_row['attribute_'.$position.'_value'] = $value;
            $position++;
        }

        $new_row['product_desc'] = $description;
        $new_row['category_desc'] = $category_desc;
        $new_row['image'] = $image_base64;

        $new_row['wc_product_id'] = $product['id'];
        $new_row['wc_date_created'] = (new DateTime($product['date_created']))->format($export_date_format);
        $new_row['wc_gtc_stock_updated_date'] = $gtc_stock_updated_date;
        $new_row['imported_from_gtc'] = empty($gtc_product_id) ? 'no' : 'yes';
        $new_row['gtc_product_id'] = $gtc_product_id;

        return array_values($new_row);
    }

    private function getTaxRate($product)
    {
        $default = '20% VAT';

        // Get tax rates if not already cached
        if ($this->tax_rates == null)
            $this->tax_rates = $this->getClient()->getTaxRates();

        // Options contains 'gtc tax id' => 'woo tax class slug'
        $tax_map = get_option(GTCSYNC_PREFIX.'vat_rates');
        $match_tax_id = array_search($product['tax_class'], $tax_map);

        if ($match_tax_id === false)
            return $default;

        // Lookup id
        $match_tax_rate = array_filter($this->tax_rates, function($rate) use ($match_tax_id){
            return $rate['id'] == $match_tax_id;
        });
        $match_tax_rate = array_values($match_tax_rate);

        if (empty($match_tax_rate))
            return $default;

        return $match_tax_rate[0]['vat_name'];
    }

    private function getWeight($product)
    {
        if (empty($product['weight']))
            return '';

        // Value we get out of Woo does not come with units.
        // Convert to kg for Goodtill.
        return wc_get_weight($product['weight'], 'g');
    }

    private function getMeta($product, $key, $default=null){

        foreach ($product['meta_data'] as $meta){
            if ($meta->key == $key)
                return $meta->value;
        }

        return $default;
    }

    private function getThumbnail($url, $size, $quality){

        $ext = pathinfo($url)['extension'];

        if ($ext == 'jpg' || $ext == 'jpeg'){
            $source = imagecreatefromjpeg($url);
        } else if ($ext == 'png'){
            $source = imagecreatefrompng($url);
        } else if ($ext == 'gif'){
            $source = imagecreatefromgif($url);
        } else {
            $this->logError(new Exception('Unsupported type for path: '.$url, 500));
            return '';
        }

        if ($source === false){
            $this->logError(new Exception('Failed to get product image from path: '.$url, 500));
            return '';
        }

        $width = imagesx($source);
        $height = imagesy($source);

        $ar = $width > $height ? $width / $height : $height / $width;

        $new_width = $width > $height ? $size : $size / $ar;
        $new_height = $width > $height ? $size / $ar : $size;

        $destination = imagecreatetruecolor($new_width, $new_height);
        $bgcolor = imagecolorallocate($destination, 255, 255, 255);
        imagefill($destination, 0, 0, $bgcolor);
        imagecopyresampled($destination, $source, 0, 0, 0, 0, $new_width, $new_height, $width, $height);

        imagedestroy($source);

        ob_start();
        imagejpeg($destination, NULL, $quality);
        $data = ob_get_contents();
        ob_end_clean();

        imagedestroy($destination);

        $mime = 'image/jpeg';
        return 'data:'.$mime.';base64,'.base64_encode($data);
    }

    /**
     * Returns whether product export is currently running
     */
    public function isInProgress(){

        $data = $this->getData();

        if ($data == null)
            return false;

        return $data['in_progress'];
    }

    /**
     * Returns whether a CSV has been created
     */
    public function hasData(){

        $data = $this->getData();
        return $data != null && $data['in_progress'] == false;
    }

    public function getData(){

        return get_option($this->data_key);
    }

    private function updateData($nonce, $in_progress, $page, $type, $images, $created_date_from, $created_date_to, $size = null){

        $option_data = [
            'nonce' => $nonce,
            'in_progress' => $in_progress,
            'page' => $page,
            'current_product' => $page * $this->products_per_page,
            'type' => $type,
            'images' => $images,
            'created_date_from' => $created_date_from,
            'created_date_to' => $created_date_to,
            'updated' => (new DateTime)->format(GTCSYNC_DATEFORMAT),
            'size' => $size
        ];

        $updated = update_option($this->data_key, $option_data, false);
        if ($updated == false)
            throw new Exception('Unable to update export temp option.');
    }

    public function exportProducts($type = null, $images = true, $created_date_from = null, $created_date_to = null){

        $debug = [
            'products_per_page' => $this->products_per_page,
            'settings' => [
                'type' => $type,
                'images' => $images,
                'image_quality' => GTCSYNC_IMAGESETTINGS,
                'created_date_from' => $created_date_from,
                'created_date_to' => $created_date_to,
            ],
        ];

        try {
            $page_products = 1;
            $fd = null;

            // If we haven't started, setup but don't export anything
            if ($this->isInProgress() == false){

                // Generate nonce on first run
                $nonce = wp_generate_password(32, false, false);
                $path = $this->buildFilePath($nonce);

                // Open handle
                $fd = fopen($path, 'a+');

                // Write header
                fputcsv($fd, $this->getHeader());
                fflush($fd);
                fclose($fd);

                $this->updateData($nonce, true, $page_products, $type, $images, $created_date_from, $created_date_to);
                return;
            }
            // Otherwise load data and continue
            else {
                $data = $this->getData();
                $nonce = $data['nonce'];
                $page_products = $data['page'];
                $type = $data['type'];
                $images = $data['images'];
                $created_date_from = $data['created_date_from'];
                $created_date_to = $data['created_date_to'];

                $path = $this->buildFilePath($nonce);

                // Open handle
                $fd = fopen($path, 'a+');
            }

            $debug['path'] = $path;
            $debug['page'] = $page_products;

            // Get products one page at a time
            $products = $this->getProducts($page_products, $type, $created_date_from, $created_date_to);

            // Stop if finished
            if (empty($products)) {

                fclose($fd);

                $size = filesize($path);

                $this->updateData($nonce, false, $page_products, $type, $images, $created_date_from, $created_date_to, $size);
                return;
            }

            foreach ($products as $product) {

                // Add main product
                $debug['product'] = $product;
                fputcsv($fd, $this->productToRow($product, null, $images));

                // Add children
                if ($product['type'] == 'variable') {

                    // Get variants one page at a time
                    $page_variants = 1;
                    while (true) {
                        $variants = $this->getVariations($product, $page_variants);

                        // Stop if none left
                        if (empty($variants))
                            break;

                        foreach ($variants as $variant) {
                            $debug['variant'] = $variant;
                            fputcsv($fd, $this->productToRow($variant, $product));
                            $debug['variant'] = null;
                        }

                        $page_variants++;
                    }
                }
            }

            $page_products++;

            // Flush
            fflush($fd);
            fclose($fd);

            // Save back csv, current page
            $this->updateData($nonce, true, $page_products, $type, $images, $created_date_from, $created_date_to);

        } catch (Throwable $e) {
            $e = new Exception('Error occurred while exporting product CSV', 500, $e);
            $this->logError($e, $debug);
            throw $e;
        }
    }

    public function cancel(){

        // Delete file
        $data = $this->getData();
        $path = $this->buildFilePath($data['nonce']);
        unlink($path);

        // Delete option
        delete_option($this->data_key);
    }

    private function buildFilePath($nonce){
        return wp_upload_dir()['basedir'].'/gtc_product_export_'.$nonce.'.csv';
    }

    public function getFilePath(){
        $data = $this->getData();
        return wp_upload_dir()['basedir'].'/gtc_product_export_'.$data['nonce'].'.csv';
    }

    public function getFileUrl()
    {
        $data = $this->getData();
        return wp_upload_dir()['baseurl'].'/gtc_product_export_'.$data['nonce'].'.csv';
    }
}