<?php

class GTCSyncImport extends GTCBase {

    const IMPORT_STATUS_IMPORTED = 'imported';
    const IMPORT_STATUS_PARTIAL = 'partial';
    const IMPORT_STATUS_NOT_IMPORTED = 'not_imported';
    const IMPORT_STATUS_UNKNOWN = 'unknown';

    private $tax_classes = null;
    private $tags = [];
    private $categories = [];
    private $brands = [];
    private $attributes = [];
    private $data_key = GTCSYNC_PREFIX.'import_temp';
    private $settings_id = GTCSYNC_ID.'-settings';

    function __construct(){

        add_action('admin_init', function() {
            register_setting($this->settings_id.'_import', $this->data_key);
        });
    }

    public function getProducts(){

        $import_status_labels = [
            static::IMPORT_STATUS_IMPORTED => 'Imported',
            static::IMPORT_STATUS_PARTIAL => 'Cannot import - Some variants already exist in WooCommerce',
            static::IMPORT_STATUS_NOT_IMPORTED => 'Not imported',
            static::IMPORT_STATUS_UNKNOWN => 'Import status unknown',
        ];

        $out = [];

        $gtc_products = $this->getClient()->getProducts();
        foreach ($gtc_products as $gtc_product){

            // Check if already imported - possibilities:
            // not_imported - does not exist in WC at all.
            // partial - parent product does not exist in WC but some variants do.
            // imported - parent product does exist.

            // Set a default value
            $gtc_product['imported'] = static::IMPORT_STATUS_UNKNOWN;

            if (empty($gtc_product['product_sku']) == false){
                // Check if the main product is imported
                $parent_woo_product_id = wc_get_product_id_by_sku($gtc_product['product_sku']);
                $parent_imported = $parent_woo_product_id != 0;

                if ($parent_imported)
                    $gtc_product['woo_product_id'] = $parent_woo_product_id;

                // Check whether the parent product or any variants are imported
                $any_imported = $parent_imported;
                foreach ($gtc_product['variants'] as $variant){
                    // If false, import_issue will be set, so no need to validate here
                    if (empty($variant['product_sku']) == false){
                        // Get wc ID product for GTC variant
                        $wc_variant_id = wc_get_product_id_by_sku($variant['product_sku']);

                        if ($wc_variant_id != 0) {
                            $any_imported = true;
                            break;
                        }
                    }
                }

                // Set status depending on imported status
                if ($any_imported == false)
                    $gtc_product['imported'] = static::IMPORT_STATUS_NOT_IMPORTED;
                else if ($parent_imported == false && $any_imported)
                    $gtc_product['imported'] = static::IMPORT_STATUS_PARTIAL;
                else
                    $gtc_product['imported'] = static::IMPORT_STATUS_IMPORTED;
            }

            // Count variants
            $gtc_product['variant_count'] = 0;
            if ($gtc_product['has_variant'])
                $gtc_product['variant_count'] = count($gtc_product['variants']);

            // Set status label
            // Check Goodtill status
            if (empty($gtc_product['import_issue']) == false)
                $gtc_product['status'] = 'Cannot import - ' . $gtc_product['import_issue'];
            // Check Wooocommerce status
            else
                $gtc_product['status'] = $import_status_labels[$gtc_product['imported']];

            // Set whether product can be imported
            $gtc_product['can_import'] = $gtc_product['imported'] == static::IMPORT_STATUS_NOT_IMPORTED &&
                empty($gtc_product['import_issue']);

            // Set whether validation prevented import
            $gtc_product['prevented_import'] = $gtc_product['imported'] == static::IMPORT_STATUS_PARTIAL ||
                empty($gtc_product['import_issue']) == false;

            // Format SKU
            $gtc_product['product_sku_html'] = $this->skuHtml($gtc_product['product_sku']);

            $out[] = $gtc_product;
        }

        return $out;
    }

    /**
     * Import a single product from the GTC API
     */
    private function importProduct($gtc_product, $status){

        $global_attributes = boolval(get_option(GTCSYNC_PREFIX.'global_attributes'));

        $debug = [
            'gtc_product' => $gtc_product,
            'import_status' => $status,
            'global_attributes' => $global_attributes,
        ];

        try {
            // Attribute name used as a dummy attribute when importing
            // variable products
            $attribute_name = 'Type';
            $attribute_slug = 'type';

            // Property list http://woocommerce.github.io/woocommerce-rest-api-docs/#product-properties
            $data = [];
            $data['name'] = $gtc_product['product_name'];
            $data['description'] = $gtc_product['product_desc'];
            $data['sku'] = $gtc_product['product_sku'];
            $data['regular_price'] = $gtc_product['selling_price'];
            $data['status'] = $status;

            // Input weight units depend on the weight unit setting in woo.
            // Weights exported from GTC in grams.
            if ($gtc_product['weight'])
                $data['weight'] = $this->convertWeight($gtc_product['weight']);

            // Category
            $cat_name = $gtc_product['category'];
            if ($cat_name) {
                try {
                    $cat_id = $this->getCategoryId($cat_name, $gtc_product['category_desc']);
                    $data['categories'] = [['id' => $cat_id]];
                } catch (\Exception $e) {
                    $e = new \Exception('Failed to get/create product category during import.', 500, $e);
                    $this->logError($e, array_merge($debug, ['category_name' => $cat_name]));
                }
            }

            // Tags
            foreach ($gtc_product['tags'] as $tag) {
                try {
                    $tag_id = $this->getTagId($tag);
                    $data['tags'][] = ['id' => $tag_id];
                } catch (\Exception $e) {
                    $e = new \Exception('Failed to get/create product tag during import.', 500, $e);
                    $this->logError($e, array_merge($debug, ['tag_name' => $tag]));
                }
            }

            // Import stock level
            $inventory_data = $this->convertInventoryData(
                $gtc_product['track_inventory'],
                $gtc_product['inventory']
            );
            $data['manage_stock'] = $inventory_data['track_inventory'];
            $data['stock_quantity'] = $inventory_data['quantity'];

            // If the woocommerce brand plugin is enabled, use that for setting
            // brands, otherwise just add it as an attribute
            if (empty($gtc_product['brand']) == false) {
                $brand_name = $gtc_product['brand'];
                $has_wc_brands = is_plugin_active('woocommerce-brands/woocommerce-brands.php');
                if ($has_wc_brands) {
                    // Find existing brand or create new, insert as brand
                    try {
                        $brand_id = $this->getBrandId($brand_name);
                        $data['brands'] = [$brand_id];
                    } catch (\Exception $e) {
                        $e = new \Exception('Failed to get/create product brand during import.', 500, $e);
                        $this->logError($e, array_merge($debug, ['brand_name' => $brand_name]));
                    }
                } else {
                    // Add as attribute
                    $data['attributes'][] = [
                        'name' => 'Brand',
                        'slug' => 'brand',
                        'position' => 100,
                        'visible' => true,
                        'variation' => false,
                        'options' => [$brand_name]
                    ];
                }
            }

            // Map GTC VAT rate with woo tax class
            $data['tax_status'] = 'taxable';
            $tax_class = $this->getTaxClass($gtc_product['vat_id']);
            if ($tax_class)
                $data['tax_class'] = $tax_class;

            // Add Goodtill metadata
            $now = (new DateTime)->format(GTCSYNC_DATEFORMAT);
            $metadata = [
                'gtc-product-id' => $gtc_product['product_id'],
                'gtc-import-date' => $now,
                'gtc-stock-updated-date' => $now,
                'gtc-import-version' => GTCSYNC_VERSION,
            ];
            foreach ($metadata as $key => $value) {
                $data['meta_data'][] = [
                    'key' => $key,
                    'value' => $value
                ];
            }

            $data['type'] = 'simple';

            // If this is a parent product with variants, get variant IDs
            if ($gtc_product['has_variant']) {

                // Fix cases where there is no variants array
                if (isset($gtc_product['variants']) == false)
                    $gtc_product['variants'] = [];

                $data['type'] = 'variable';
                $variant_products = $gtc_product['variants'];

                // Using custom attributes
                if ($gtc_product['has_attributes']) {

                    // Build array of attribute_name => [attribute_value...]
                    $attribute_values = [];
                    foreach ($gtc_product['variants'] as $variant) {
                        foreach ($variant['attributes'] as $attribute) {
                            // Add, remove duplicates
                            $name = $attribute['name'];
                            $attribute_values[$name][] = $attribute['value'];
                            $attribute_values[$name] = array_values(array_unique($attribute_values[$name]));
                        }
                    }

                    // Add each attribute
                    foreach ($gtc_product['attributes'] as $attribute) {

                        // Get values, if any
                        $options = [];
                        if (isset($attribute_values[$attribute['name']]))
                            $options = $attribute_values[$attribute['name']];

                        // If we're using global attributes and we can find a matching WC global attribute,
                        // add the name
                        if ($global_attributes && $this->getAttributeId($attribute['name']) != null){
                            $data['attributes'][] = [
                                'id' => $this->getAttributeId($attribute['name']),
                                'position' => 0,
                                'visible' => true,
                                'variation' => true,
                                'options' => $options,
                            ];
                        }
                        // Otherwise add a new local attribute
                        else {
                            $data['attributes'][] = [
                                'name' => $attribute['name'],
                                'slug' => sanitize_title($attribute['name']),
                                'position' => 0,
                                'visible' => true,
                                'variation' => true,
                                'options' => $options,
                            ];
                        }
                    }
                }
                // Using single level of variant
                else {

                    // Get the variant names
                    $variant_product_names = array_map(function ($product) {
                        return $product['product_name'];
                    }, $variant_products);
                    $variant_product_names = array_values($variant_product_names);

                    // Add our generic 'subtype' attribute
                    $data['attributes'][] = [
                        'name' => $attribute_name,
                        'slug' => $attribute_slug,
                        'position' => 0,
                        'visible' => true,
                        'variation' => true,
                        'options' => $variant_product_names
                    ];
                }

                // Add each of our variations
                foreach ($variant_products as $variant) {

                    $inventory_data = $this->convertInventoryData(
                        $variant['track_inventory'],
                        $variant['inventory']
                    );

                    $variant_data = [
                        'description' => $variant['product_desc'],
                        'manage_stock' => $inventory_data['track_inventory'],
                        'stock_quantity' => $inventory_data['quantity'],
                        'sku' => $variant['product_sku'],
                        'regular_price' => $variant['selling_price'],
                        'tax_status' => 'taxable',
                        'menu_order' => $variant['display_order']
                    ];

                    if ($variant['weight'])
                        $variant_data['weight'] = $this->convertWeight($variant['weight']);

                    $tax_class = $this->getTaxClass($variant['vat_id']);
                    if ($tax_class)
                        $variant_data['tax_class'] = $tax_class;

                    $attributes = [];
                    // Add attributes if GTC product has them
                    if ($gtc_product['has_attributes']) {
                        $attributes = array_map(function($attr) use ($global_attributes, $gtc_product) {

                            // If we're using global attributes and we can find a matching WC global attribute,
                            // set the ID.
                            if ($global_attributes && $this->getAttributeId($attr['name'])){
                                return [
                                    'id' => $this->getAttributeId($attr['name']),
                                    'option' => $attr['value'],
                                ];
                            }

                            // Otherwise, we'll pass the name to make a new local attribute.

                            // First check that the value does not contain a pipe.
                            if (strpos($attr['value'], '|') !== false) {
                                $msg = htmlentities(sprintf(
                                    'Product "%s" (ID %s) has a pipe in attribute "%s" - "%s". '.
                                    'Please create a global attribute named "%s" in WooCommerce, '.
                                    'enable global attributes in the Goodtill plugin and try again.',
                                    $gtc_product['product_name'],
                                    $gtc_product['product_id'],
                                    $attr['name'],
                                    $attr['value'],
                                    $attr['name']
                                ));
                                throw new Exception($msg, 422);
                            }

                            return [
                                'name' => $attr['name'],
                                'slug' => sanitize_title($attr['name']),
                                'option' => $attr['value'],
                            ];
                        }, $variant['attributes']);
                    }
                    // Otherwise attach dummy attributes for variant only
                    else {
                        $attributes = [
                            [
                                'name' => $attribute_name,
                                'slug' => $attribute_slug,
                                'option' => $variant['product_name']
                            ]
                        ];
                    }
                    $variant_data['attributes'] = $attributes;

                    // Add Goodtill metadata
                    $now = (new DateTime)->format(GTCSYNC_DATEFORMAT);
                    $metadata = [
                        'gtc-product-id' => $variant['product_id'],
                        'gtc-import-date' => $now,
                        'gtc-stock-updated-date' => $now,
                        'gtc-import-version' => GTCSYNC_VERSION,
                    ];
                    foreach ($metadata as $key => $value) {
                        $variant_data['meta_data'][] = [
                            'key' => $key,
                            'value' => $value
                        ];
                    }

                    $data['variations'][] = $variant_data;
                }
            }

            $debug['import_product_data'] = $data;

            // Create product
            $this->getLogger()->info('Importing GTC product with id: ' . $gtc_product['product_id']);
            $woo_product = $this->createProduct($data);
            $this->getLogger()->info('Imported product has id: ' . $woo_product['id']);

            // Add variations after creating the product
            if (isset($data['variations'])) {
                foreach ($data['variations'] as $variation) {
                    $this->getLogger()->info('Adding variance to product with id: ' . $woo_product['id']);
                    $this->addProductVariation($woo_product['id'], $variation);
                }
            }
        } catch (Throwable $e) {
            $e = new Exception('Failed to import product', 500, $e);
            $this->logError($e, $debug);
            throw $e->getPrevious();
        }
    }

    public function cancel(){
        delete_option($this->data_key);
    }

    /**
     * Returns whether product export is currently running
     */
    public function isInProgress(){

        $data = $this->getData();

        if ($data == null)
            return false;

        return $data['in_progress'];
    }

    public function getData(){

        return get_option($this->data_key);
    }

    private function updateData($in_progress, $index, $products = null, $status = null){

        $option_data = $this->getData();

        $option_data['in_progress'] = $in_progress;
        $option_data['index'] = $index;
        $option_data['updated'] = (new DateTime)->format(GTCSYNC_DATEFORMAT);

        if ($status)
            $option_data['status'] = $status;

        if ($products)
            $option_data['products'] = $products;

        $updated = update_option($this->data_key, $option_data, false);
        if ($updated == false)
            throw new Exception('Unable to update import temp option.');
    }

    public function importProducts($product_ids = null, $status = 'publish'){

        $index = 0;
        $gtc_products = [];
        $import_status = $status;
        $product_batch_count = intval(get_option(GTCSYNC_PREFIX.'import_product_batch_count'));

        // If we haven't started, store our metadata in the DB
        if ($this->isInProgress() == false){

            $gtc_products = $this->getProducts();

            // Filter importable
            $gtc_products = array_filter($gtc_products, function($product) use ($product_ids){
                return $product['can_import'] && in_array($product['product_id'], $product_ids);
            });
            $gtc_products = array_values($gtc_products);

            $this->getLogger()->info('Starting import of '.count($gtc_products).' products');

            // Save temp data
            $this->updateData(true, 0, $gtc_products, $status);

            return;
        }
        // Otherwise load our data
        else {
            $data = $this->getData();
            $index = $data['index'];
            $gtc_products = $data['products'];
            $import_status = $data['status'];
        }

        $imported = 0;

        // Iterate products by index
        while ($index < count($gtc_products)) {

            $gtc_product = $gtc_products[$index];

            // Increment index for next product
            $index++;

            // Skip no SKU
            if ($gtc_product['product_sku'] == null)
                continue;

            // Skip already imported
            $woo_product_id = wc_get_product_id_by_sku($gtc_product['product_sku']);
            if ($woo_product_id != 0) {
                $this->getLogger()->info($gtc_product['product_sku'].' already imported - id '.$woo_product_id);
                continue;
            }

            // Import
            $this->getLogger()->info('Adding '.$gtc_product['product_name']);
            $this->importProduct($gtc_product, $import_status);

            // Increment imported count
            $imported++;

            $this->updateData(true, $index);

            // If we've imported everything for this batch, update and stop
            if ($imported >= $product_batch_count)
                return;
        }

        // Finished
        $this->updateData(false, $index);
    }

    private function convertWeight($value){
        $woo_weight_units = strtolower(get_option('woocommerce_weight_unit'));
        return wc_get_weight($value, $woo_weight_units, 'g');
    }

    private function createProduct($data){
        $wp_rest_request = new WP_REST_Request('POST');
        $wp_rest_request->set_body_params($data);
        $products_controller = new WC_REST_Products_Controller;
        $product_response = @$products_controller->create_item($wp_rest_request);
        $this->handleWpError($product_response);
        return $product_response->get_data();
    }

    private function addProductVariation($product_id, $variation){
        $wp_rest_request = new WP_REST_Request('POST');
        $wp_rest_request->set_body_params($variation);
        $wp_rest_request['product_id'] = $product_id;
        $variations_controller = new WC_REST_Product_Variations_Controller;
        $variation_response = @$variations_controller->create_item($wp_rest_request);
        $this->handleWpError($variation_response);
        return $variation_response->get_data();
    }

    private function getAttributeId($name){

        $name = sanitize_text_field($name);

        // Try cache
        if (isset($this->attributes[$name]))
            return $this->attributes[$name];

        // Search brands
        $data = [
            'search' => $name
        ];

        $wp_rest_request = new WP_REST_Request('GET');
        $wp_rest_request->set_query_params($data);
        $attributes_controller = new WC_REST_Product_Attributes_Controller();
        $attributes_response = @$attributes_controller->get_items($wp_rest_request);
        $this->handleWpError($attributes_response);
        $attributes = $attributes_response->get_data();

        // Return matching ID
        foreach ($attributes as $attribute) {
            if ($attribute['name'] == $name){
                $this->attributes[$name] = $attribute['id'];
                return $attribute['id'];
            }
        }

        return null;
    }

    private function getOrCreateTerm($taxonomy, $name, $description=''){
        // Search existing
        $term = term_exists($name, $taxonomy);

        // Return matching ID
        if ($term != null)
            return $term['term_id'];

        // Otherwise, create brand
        $term = wp_insert_term($name, $taxonomy, ['description' => $description]);
        $this->handleWpError($term);
        return $term['term_id'];
    }

    private function getBrandId($name){

        $taxonomy = 'product_brand';

        // Try cache
        if (isset($this->brands[$name]))
            return $this->brands[$name];

        // Get or create
        $term_id = $this->getOrCreateTerm($taxonomy, $name);
        $this->brands[$name] = $term_id;

        return $term_id;
    }

    private function getCategoryId($name, $description){

        $taxonomy = 'product_cat';

        // Try cache
        if (isset($this->categories[$name]))
            return $this->categories[$name];

        // Get or create
        $term_id = $this->getOrCreateTerm($taxonomy, $name, $description);
        $this->categories[$name] = $term_id;

        return $term_id;
    }

    private function getTagId($name){

        $taxonomy = 'product_tag';

        // Try cache
        if (isset($this->tags[$name]))
            return $this->tags[$name];

        // Get or create
        $term_id = $this->getOrCreateTerm($taxonomy, $name);
        $this->tags[$name] = $term_id;

        return $term_id;
    }

    private function getTaxClass($vat_id){

        if ($this->tax_classes == null){
            $wp_rest_request = new WP_REST_Request('GET');
            $wp_rest_request->set_query_params([]);
            $tax_class_controller = new WC_REST_Tax_Classes_Controller;
            $this->tax_classes = $tax_class_controller->get_items($wp_rest_request)->get_data();
        }

        // Get tax class slug for this vat_id
        $vat_rates = get_option(GTCSYNC_PREFIX.'vat_rates');

        if (isset($vat_rates[$vat_id]) == false){
            $this->logError(new Exception('No vat_id '.$vat_id.' in mapping', 500));
            return null;
        }

        $tax_class_slug = $vat_rates[$vat_id];

        // Get name from slug
        $tax_class_matches = array_values(array_filter($this->tax_classes, function($class) use ($tax_class_slug){
            return $class['slug'] == $tax_class_slug;
        }));

        return $tax_class_matches[0]['name'];
    }
}