<?php

class GTCSyncSettings extends GTCBase {

    private $settings_id;

    public function __construct() {

        $this->settings_id = GTCSYNC_ID.'-settings';

        add_action('admin_menu', function() {
            add_options_page(
                GTCSYNC_NAME.' Settings',
                GTCSYNC_NAME,
                'manage_options',
                $this->settings_id,
                [$this, 'settingsPage']
            );
        });

        add_action('admin_init', function() {
            register_setting($this->settings_id.'_eula', GTCSYNC_PREFIX.'eula_version');
            register_setting($this->settings_id.'_eula', GTCSYNC_PREFIX.'eula_datetime');
            register_setting($this->settings_id.'_eula', GTCSYNC_PREFIX.'eula_user');
            register_setting($this->settings_id.'_credentials', GTCSYNC_PREFIX.'subdomain');
            register_setting($this->settings_id.'_credentials', GTCSYNC_PREFIX.'username');
            register_setting($this->settings_id.'_credentials', GTCSYNC_PREFIX.'password');
            register_setting($this->settings_id.'_credentials', GTCSYNC_PREFIX.'token');
            register_setting($this->settings_id.'_tax', GTCSYNC_PREFIX.'vat_rates');
            register_setting($this->settings_id.'_import_export', GTCSYNC_PREFIX.'global_attributes');
            register_setting($this->settings_id.'_import_export', GTCSYNC_PREFIX.'import_product_batch_count');
            register_setting($this->settings_id.'_sync', GTCSYNC_PREFIX.'sync_enabled');
            register_setting($this->settings_id.'_sync', GTCSYNC_PREFIX.'sync_adjustment_enabled');
            register_setting($this->settings_id.'_sync', GTCSYNC_PREFIX.'sync_cache_seconds');
            register_setting($this->settings_id.'_background_sync', GTCSYNC_PREFIX.'background_sync_enabled');
            register_setting($this->settings_id.'_background_sync', GTCSYNC_PREFIX.'background_sync_interval_seconds');
            register_setting($this->settings_id.'_background_sync', GTCSYNC_PREFIX.'background_sync_flush_cache');

            // Defaults
            $defaults = [
                'global_attributes' => false,
                'import_product_batch_count' => 5,
                'sync_enabled' => false,
                'sync_adjustment_enabled' => true,
                'sync_cache_seconds' => 3 * 60,
                'background_sync_enabled' => true,
                'background_sync_interval_seconds' => 30 * 60,
                'background_sync_flush_cache' => '',
            ];

            foreach ($defaults as $setting_key => $setting_default){
                $full_settings_key = GTCSYNC_PREFIX.$setting_key;
                $current_value = get_option($full_settings_key, null);
                if ($current_value === null)
                    update_option($full_settings_key, $setting_default);
            }

            // User enters value in minutes which we'll convert to seconds
            // before commit. This allows changing the user-facing units to
            // seconds later on if necessary.
            add_filter('pre_update_option_'.GTCSYNC_PREFIX.'sync_cache_seconds', function($new_value, $old_value) {
                return intval($new_value) * 60;
            }, 10, 2);

            add_filter('pre_update_option_'.GTCSYNC_PREFIX.'background_sync_interval_seconds', function($new_value, $old_value) {
                return intval($new_value) * 60;
            }, 10, 2);

            // Turn checkbox 'on' or null value in to boolean.
            add_filter('pre_update_option_'.GTCSYNC_PREFIX.'global_attributes', function($new_value, $old_value) {
                return empty($new_value) == false;
            }, 10, 2);

            add_filter('pre_update_option_'.GTCSYNC_PREFIX.'sync_enabled', function($new_value, $old_value) {
                return empty($new_value) == false;
            }, 10, 2);

            add_filter('pre_update_option_'.GTCSYNC_PREFIX.'sync_adjustment_enabled', function($new_value, $old_value) {
                return empty($new_value) == false;
            }, 10, 2);

            add_filter('pre_update_option_'.GTCSYNC_PREFIX.'background_sync_enabled', function($new_value, $old_value) {

                // Probably not what pre_update is meant for but it works.
                // Unregister the cron job now in case it will be disabled or
                // the interval will change. Prevents having to wait an additional
                // cycle of the job.
                GTCSyncCron::unregister();

                return empty($new_value) == false;
            }, 10, 2);

            // Save EULA settings
            add_filter('pre_update_option_'.GTCSYNC_PREFIX.'eula_user', function($new_value, $old_value) {

                update_option(GTCSYNC_PREFIX.'eula_version', GTCSYNC_VERSION);
                update_option(GTCSYNC_PREFIX.'eula_datetime', (new DateTime)->format(GTCSYNC_DATEFORMAT));

                return get_current_user_id();
            }, 10, 2);

            // Clear vat rate / tax class mapping if store changed
            add_filter('pre_update_option_'.GTCSYNC_PREFIX.'subdomain', function($new_value, $old_value) {

                if (empty($old_value) == false && $new_value != $old_value){
                    update_option(GTCSYNC_PREFIX.'vat_rates', null);
                    $this->getLogger()->info('Cleared vat rates - subdomain changed');
                }

                return $new_value;
            }, 10, 2);
        });
    }

    public function settingsPage() {

        // Only show EULA if not accepted
        if ($this->acceptedEula() == false){
            $this->eulaPage();
            return;
        }

       ?><div class="wrap">
            <h2><?php echo GTCSYNC_NAME; ?></h2>
       <?php

        $live_sync_enabled = get_option(GTCSYNC_PREFIX.'sync_enabled');
        $background_sync_enabled = get_option(GTCSYNC_PREFIX.'background_sync_enabled');

        // WooCommerce disabled warning
        if ($this->woocommerceActive() == false){
           ?><div class="wrap">
                <div class="error notice">
                    <p>Please enable WooCommerce on the<a href="plugins.php?">plugins page</a>.</p>
                </div>
            </div>
            <?php
            return;
        }

        // Show cache warning
        if ($this->hasCachePlugin() && $live_sync_enabled && $background_sync_enabled == false){
            ?>
            <div class="error notice">
                <p><strong>Using a cache plugin?</strong>
                Cache plugins may interfere with live stock sync - you are advised to enable
                background stock sync to ensure that stock data is kept up to date. You can
                safely ignore this warning if product pages are not cached.</p>
            </div>
            <?php
        }

        // Show now stock syncing warning
        $no_stock_sync_enabled = $live_sync_enabled == false && $background_sync_enabled == false;
        if ($no_stock_sync_enabled && $this->isConfigured()){
            ?>
            <div class="error notice">
                <p><strong>Stock syncing disabled</strong>
                Enable live and/or background stock sync to start syncing stock values from Goodtill.</p>
            </div>
            <?php
        }

        // WooCommerce stock management disabled warning
        if (get_option('woocommerce_manage_stock') === 'no'){
            ?>
            <div class="error notice"><p>WooCommerce stock management is disabled.
                <a href="admin.php?page=wc-settings&tab=products&section=inventory">Enable</a> this to
                show the inventory count synced from Goodtill.</p>
            </div>
            <?php
        }

        $current_tab = gtc_sync_get('tab');

        $tabs = [
            [
                'id' => 'credentials',
                'name' => 'Credentials',
            ],
            [
                'id' => 'tax',
                'name' => 'Tax',
            ],
            [
                'id' => 'import_export',
                'name' => 'Import & Export',
            ],
            [
                'id' => 'live_sync',
                'name' => 'Live Sync',
            ],
            [
                'id' => 'background_sync',
                'name' => 'Background Sync',
            ],
            [
                'id' => 'support',
                'name' => 'Support & EULA',
            ]
        ];

        ?><h2 class="nav-tab-wrapper">
            <?php foreach ($tabs as $index => $tab) { ?>
                <a href="<?php echo esc_url(add_query_arg('tab', $tab['id'])); ?>" class="nav-tab
                    <?php if (($current_tab == '' && $index == 0) || $current_tab == $tab['id']) { echo 'nav-tab-active'; } ?>">
                    <?php echo htmlentities($tab['name']); ?>
                </a>
            <?php } ?>
        </h2>
        <div class="wrap"><?php

        if ($current_tab == '' || $current_tab == 'credentials'){
            $this->credentialsTab();
        } else if ($current_tab == 'tax') {
            $this->taxClassTab();
        } else if ($current_tab == 'import_export') {
            $this->importExportTab();
        } else if ($current_tab == 'live_sync') {
            $this->liveSyncTab();
        } else if ($current_tab == 'background_sync') {
            $this->backgroundSyncTab();
        } else if ($current_tab == 'support') {
            $this->supportTab();
        }

        ?></div><?php
    }

    function eulaPage(){

        $this->eulaContent();
        ?>
        <form method="POST" action="options.php">
            <input type="hidden" name="action" value="admin_post_eula_accept" />
            <?php
            $settings_id = $this->settings_id.'_eula';
            settings_fields($settings_id);
            do_settings_sections($settings_id);
            submit_button('I accept', 'primary', GTCSYNC_PREFIX.'eula_user'); ?>
        </form>
        <?php
    }

    public function supportTab()
    {
        // Force login which sends up to date debug data
        $login_successful = false;
        if ($this->isConfigured()){
            try {
                $this->getClient()->login();
                $login_successful = true;
            } catch (\Exception $e) {

            }
        }

        $system_info = $this->getSystemInfo();
        $now = (new \DateTime('now', new \DateTimeZone('Europe/London')))->format(GTCSYNC_DATEFORMAT);

        $debug_data = [
            'goodtill' => [
                'subdomain' => get_option(GTCSYNC_PREFIX . 'subdomain'),
                'username' => get_option(GTCSYNC_PREFIX . 'username'),
                'outlet_id' => get_option(GTCSYNC_PREFIX . 'outlet_id'),
                'outlet_name' => get_option(GTCSYNC_PREFIX . 'outlet_name'),
                'login_successful' => $login_successful,
            ],
            'versions' => [
                'version' => $system_info['plugin_version'],
                'php_version' => $system_info['php_version'],
                'wordpress_version' => $system_info['wordpress_version'],
                'woocommerce_version' => $system_info['woocommerce_version'],
            ],
            'options' => [
                'global_attributes' => boolval(get_option(GTCSYNC_PREFIX . 'global_attributes')),
                'import_product_batch_count' => intval(get_option(GTCSYNC_PREFIX . 'import_product_batch_count')),
                'sync_enabled' => boolval(get_option(GTCSYNC_PREFIX . 'sync_enabled')),
                'sync_adjustment_enabled' => boolval(get_option(GTCSYNC_PREFIX . 'sync_adjustment_enabled')),
                'sync_cache_seconds' => intval(get_option('gtcsync_sync_cache_seconds')),
                'background_sync_enabled' => boolval(get_option('gtcsync_background_sync_enabled')),
                'background_sync_interval_seconds' => intval(get_option('gtcsync_background_sync_interval_seconds')),
                'background_sync_last_synced' => get_option('gtcsync_background_sync_last_synced'),
                'background_sync_flush_cache' => get_option('gtcsync_background_sync_flush_cache'),
            ],
            'sync' => [
                'next_background_sync' => wp_next_scheduled(GTCSyncCron::HOOK_NAME),
            ],
            'cron' => $system_info['cron'],
            'timezone' => $system_info['timezone'],
            'plugins' => $this->getActivePlugins(),
            'env' => [
                'debug' => GTCSYNC_DEBUG,
                'api_ip' => GTCSYNC_IP,
                'api_url' => GTCSYNC_URL,
                'app_url' => GTCSYNC_APP,
                'image_settings' => GTCSYNC_IMAGESETTINGS,
                'bugsnag_key' => '...'.mb_substr(GTCSYNC_BUGSNAG_KEY, 24),
                'bugsnag_release_stage' => GTCSYNC_BUGSNAG_RELEASE_STAGE,
            ],
            'meta' => [
                'url' => get_site_url(),
                'time' => $now,
            ]
        ];

        $this->eulaContent();
        ?><br>
        <h2>Support</h2>
        Documentation for this plugin is available on our support portal:<br>
        <a href="https://thegoodtill.com/support/woocommerce/" target="_blank" class="button button-primary">
            Visit support portal
        </a>
        <br><br>
        <h2>Debug</h2>
        <p>The following details can be provided to Goodtill staff to aid support queries.</p>
        <textarea readonly style="width: 100%; min-height: 700px;"><?php echo json_encode($debug_data, JSON_PRETTY_PRINT); ?></textarea> <?php
    }

    public function eulaContent(){
        ?>
        <style>
            #gtcsync-table-eula {
                text-align: left;
            }

            #gtcsync-table-eula th {
                vertical-align: top;
                white-space: nowrap;
                padding-right: 10px;
            }
        </style>
        <h3 style="font-size: 2em; line-height: 1em;">Goodtill Stock Sync - End User License Agreement</h3>
        <p>This plugin will periodically send diagnostic data in order to
            aid support queries and inform us about which systems and configurations
            we need to support.</p>
        <p>This data includes:</p>
        <table id="gtcsync-table-eula">
            <tr>
                <th>Error data</th>
                <td>Used to aid support queries and make the plugin more robust.
                    Includes error messages, related data (eg product data) and page URL.</td>
            </tr>
            <tr>
                <th>System information</th>
                <td>Used to inform us about the system configurations we need to
                    support. Includes data such as PHP version, Wordpress version etc.</td>
            </tr>
            <tr>
                <th>Plugin settings</th>
                <td>Used to aid support queries and inform us about which features
                    are being used.</td>
            </tr>
            <tr>
                <th>Installed plugins</th>
                <td>Used to inform us about which plugins we may need to integrate
                    with in the future.</td>
            </tr>
        </table>
        <h3>No customer or store admin user data is ever sent from this plugin.</h3>
        <?php
        $accepted_date = get_option(GTCSYNC_PREFIX.'eula_datetime');
        $user_id = get_option(GTCSYNC_PREFIX.'eula_user');
        if (empty($accepted_date) == false) {
            printf(
                '<p>Accepted by %s on %s</p>',
                get_user_by('id', $user_id)->display_name,
                $this->setTimeZone($accepted_date)
            );
        }
    }

    public function credentialsTab(){

        $client = $this->getClient();
        $accepted = false;
        $outlet_name = null;

        if ($client->hasCredentials()){
            try {
                $data = $client->login();
                $accepted = true;
                $outlet_name = get_option(GTCSYNC_PREFIX.'outlet_name');
                if ($this->hasTaxClasses()){ ?>
                    <div class="updated notice"><p>
                        Credentials accepted.
                        <a href="admin.php?page=gtc-stock-sync-admin">You can now import or export your Goodtill products</a>.
                    </p></div>
                <?php } else { ?>
                    <div class="updated notice"><p>
                        Credentials accepted.
                        <a href="admin.php?page=gtc-stock-sync-settings&tab=tax">You can now map vat rates to tax classes</a>.
                    </p></div>
                <?php }
            } catch (Throwable $e){
                echo '<div class="error notice"><p>'.$e->getMessage().'</p></div>';
            }
        }

        ?>
        <p>Please login using the credentials for an Administrator user in your Goodtill store.<br>
        If your store has multiple outlets, please enter the username for a user assigned to the outlet
        that you want the stock to be taken from.</p>
        <form method="POST" action="options.php">
            <?php
                $settings_id = $this->settings_id.'_credentials';
                settings_fields($settings_id);
                do_settings_sections($settings_id);
            ?>
            <table class="form-table">
                <tr>
                    <th scope="row"><label>Store subdomain</label></th>
                    <td><input type="text" name="<?php echo GTCSYNC_PREFIX.'subdomain'; ?>"
                            value="<?php echo esc_attr(get_option(GTCSYNC_PREFIX.'subdomain')); ?>" size="50">
                    </td>
                </tr>
                <tr>
                    <th scope="row"><label>Username</label></th>
                    <td><input type="text" name="<?php echo GTCSYNC_PREFIX.'username'; ?>"
                            value="<?php echo esc_attr(get_option(GTCSYNC_PREFIX.'username')); ?>" size="50">
                    </td>
                </tr>
                <tr>
                    <th scope="row"><label>Password</label></th>
                    <td><input type="password" name="<?php echo GTCSYNC_PREFIX.'password'; ?>"
                            value="<?php echo esc_attr(get_option(GTCSYNC_PREFIX.'password')); ?>" size="50">
                    </td>
                </tr>
                <?php if ($accepted && empty($outlet_name) == false) { ?>
                    <th scope="row"><label>Outlet</label></th>
                    <td><input type="text" readonly value="<?php echo esc_attr($outlet_name); ?>" size="50"></td>
                <?php } ?>
            </table>
            <?php
                submit_button();
            ?>
        </form>
        <?php
    }

    function taxClassTab(){

        $has_tax_rates = false;
        if ($this->isConfigured()){
            try {
                $tax_rates = $this->getClient()->getTaxRates();
                $has_tax_rates = true;
            } catch (Throwable $e){
                $e = new Exception('Could not get Goodtill tax rates', 500, $e);
                $this->logError($e);
                $tax_rates = [];
            }
        }

        $wp_rest_request = new WP_REST_Request('GET');
        $wp_rest_request->set_query_params([]);
        $tax_class_controller = new WC_REST_Tax_Classes_Controller;
        $tax_classes = $tax_class_controller->get_items($wp_rest_request)->get_data();

        $vat_rates = get_option(GTCSYNC_PREFIX.'vat_rates');
        ?>
        <p>This page allows you to map each Goodtill VAT rate to a corresponding WooCommerce Tax Class.<br>
            This mapping will be used when importing and exporting products.</p>
        <form method="POST" action="options.php">
            <?php
                $settings_id = $this->settings_id.'_tax';
                settings_fields($settings_id);
                do_settings_sections($settings_id);
            ?>
            <table class="form-table">
            <?php if ($has_tax_rates) { ?>
                <tr>
                    <th>Goodtill VAT Rate</th>
                    <td><strong>WooCommerce Tax Class</strong></td>
                </tr>
                <?php foreach ($tax_rates as $tax_rate) { ?>
                    <tr>
                        <th scope="row"><label><?php echo htmlspecialchars($tax_rate['vat_name']); ?></label></th>
                        <td>
                            <select name="<?php echo GTCSYNC_PREFIX.'vat_rates['.$tax_rate['id'].']'; ?>">
                                <?php foreach ($tax_classes as $tax_class) { ?>
                                    <option value="<?php echo $tax_class['slug']; ?>"
                                        <?php
                                            $key = $tax_rate['id'];
                                            $selected = isset($vat_rates[$key]) ? $vat_rates[$key] : '';
                                            if ($selected == $tax_class['slug'])
                                                echo 'selected';
                                        ?>>
                                        <?php echo $tax_class['name']; ?>
                                    </option>
                                <?php } ?>
                            </select>
                        </td>
                    </tr>
                <?php } ?>
            <?php } else { ?>
                <p>Tax rates are available after entering valid credentials.</p>
            <?php } ?>
            </table>
            <?php
                if ($has_tax_rates)
                    submit_button();
            ?>
        </form>
        <?php
    }

    function importExportTab(){

        $global_attributes_enabled = get_option(GTCSYNC_PREFIX.'global_attributes');
        $import_product_batch_count = get_option(GTCSYNC_PREFIX.'import_product_batch_count');

        ?>
        <div class="wrap">
            <form method="POST" action="options.php">
                <?php
                    $settings_id = $this->settings_id.'_import_export';
                    settings_fields($settings_id);
                    do_settings_sections($settings_id);
                ?>
                <table class="form-table">
                    <tr>
                        <th scope="row"><label>Use global attributes on import</label></th>
                        <td>
                            <label>
                                <input type="checkbox" name="<?php echo GTCSYNC_PREFIX.'global_attributes'; ?>"
                                    <?php echo $global_attributes_enabled ? 'checked' : ''; ?>>
                                Enable
                            </label>
                            <p class="description">If enabled, imported product attributes will be mapped to existing
                            attributes in your store if a match can be found based on name. Otherwise, local product
                            attributes will be used.</p>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row"><label>Import product batch count</label></th>
                        <td>
                            <input type="number" name="<?php echo GTCSYNC_PREFIX.'import_product_batch_count'; ?>"
                                value="<?php echo $import_product_batch_count; ?>" min="1" max="25">
                            <p class="description">Number of products to import per request. Set lower when importing
                            large numbers of variants.</p>
                        </td>
                    </tr>
                </table>
                <?php submit_button(); ?>
            </form>
        </div><?php
    }

    function liveSyncTab(){

        $live_sync_enabled = get_option(GTCSYNC_PREFIX.'sync_enabled');
        $sync_adjustment_enabled = get_option(GTCSYNC_PREFIX.'sync_adjustment_enabled');

        $sync_cache_seconds = get_option(GTCSYNC_PREFIX.'sync_cache_seconds');
        $sync_cache_minutes = $sync_cache_seconds / 60;
        ?>
        <div class="wrap">

            <form method="POST" action="options.php">
                <?php
                    $settings_id = $this->settings_id.'_sync';
                    settings_fields($settings_id);
                    do_settings_sections($settings_id);
                ?>
                <table class="form-table">
                    <tr>
                        <th scope="row"><label>Enable live stock sync</label></th>
                        <td>
                            <label>
                                <input type="checkbox" name="<?php echo GTCSYNC_PREFIX.'sync_enabled'; ?>"
                                    <?php echo $live_sync_enabled ? 'checked' : ''; ?>>
                                Enable
                            </label>
                            <p class="description">Enable updating the WooCommerce stock quantity for products when they are
                                viewed by customers (if cache period has elapsed).<br>
                                <strong>Not recommended - use background sync instead when possible.</strong></p>

                        </td>
                    </tr>
                    <tr>
                        <th scope="row"><label>Enable stock adjustment sync</label></th>
                        <td>
                            <label>
                                <input type="checkbox" name="<?php echo GTCSYNC_PREFIX.'sync_adjustment_enabled'; ?>"
                                    <?php echo $sync_adjustment_enabled ? 'checked' : ''; ?>>
                                Enable
                            </label>
                            <p class="description">Enable decrementing the stock in Goodtill when
                                orders are completed in WooCommerce and increment the stock if orders are returned or cancelled.</p>
                        </td>
                    </tr>
                    <tr>
                        <th scope="row"><label>Cache stock quantity (minutes)</label></th>
                        <td>
                            <input type="number" name="<?php echo GTCSYNC_PREFIX.'sync_cache_seconds'; ?>"
                                value="<?php echo $sync_cache_minutes; ?>" min="1" max="120">
                            <p class="description">Stock will not be synced multiple times within this time
                                period.<br>
                                Setting this value lower will increase the accuracy of the stock quantity at
                                the expense of increasing the page load time when a user views a product.</p>
                        </td>
                    </tr>
                </table>
                <?php submit_button(); ?>
            </form>
        </div><?php
    }

    function backgroundSyncTab(){

        $background_sync_enabled = get_option(GTCSYNC_PREFIX.'background_sync_enabled');

        $flush_cache = get_option(GTCSYNC_PREFIX.'background_sync_flush_cache');

        $sync_interval_seconds = get_option(GTCSYNC_PREFIX.'background_sync_interval_seconds');
        $sync_interval_minutes = $sync_interval_seconds / 60;

        $last_sync = get_option(GTCSYNC_PREFIX.'background_sync_last_synced');
        if (empty($last_sync) == false)
            $last_sync = $this->setTimeZone($last_sync);

        ?>
        <div class="wrap">

        <form method="POST" action="options.php">
            <?php
            $settings_id = $this->settings_id.'_background_sync';
            settings_fields($settings_id);
            do_settings_sections($settings_id);
            ?>
            <table class="form-table">
                <tr>
                    <th scope="row"><label>Enable background stock sync</label></th>
                    <td>
                        <label>
                            <input type="checkbox" name="<?php echo GTCSYNC_PREFIX.'background_sync_enabled'; ?>"
                                   <?php echo $background_sync_enabled ? 'checked' : ''; ?>>
                            Enable
                        </label>
                        <p class="description">Enable background syncing of stock for all published products at a set interval.<br>
                        This setting does not affect 'live syncing' of stock.<br>
                        Please note that this feature makes use of
                        <a href="https://developer.wordpress.org/plugins/cron/">WP Cron</a> which has some
                        additional requirements.</p>
                    </td>
                </tr>
                <tr>
                    <th scope="row"><label>Background stock sync interval (minutes)</label></th>
                    <td>
                        <input type="number" name="<?php echo GTCSYNC_PREFIX.'background_sync_interval_seconds'; ?>"
                               value="<?php echo $sync_interval_minutes; ?>" min="1" max="2400">
                        <p class="description">The interval between each background stock sync.<br>
                        Please note that not all products will be synced at every interval if you
                        have a large catalogue of products.</p>
                    </td>
                </tr>
                <tr>
                    <th scope="row"><label>Flush cache after update</label></th>
                    <td>
                        <select name="<?php echo GTCSYNC_PREFIX.'background_sync_flush_cache'; ?>">
                            <option value="" <?php echo $flush_cache == '' ? 'selected' : ''; ?>>Disabled</option>
                            <option value="product_posts" <?php echo $flush_cache == 'product_posts' ? 'selected' : ''; ?>>Product post pages</option>
                            <option value="all" <?php echo $flush_cache == 'all' ? 'selected' : ''; ?>>All pages</option>
                        </select>
                        <p class="description">Remove pages from the cache after completing the stock
                            update for all products.<br>
                            This prevents showing an outdated stock value or in/out of stock status when
                            using cache plugins.<br>
                            Supported cache plugins: <?php echo implode(', ', $this->supportCachePlugins()); ?>.</p>
                    </td>
                </tr>
                <tr>
                    <th scope="row">Next scheduled sync</th>
                    <td><?php echo GTCSyncCron::nextSync(); ?></td>
                </tr>
                <tr>
                    <th scope="row">Last sync</th>
                    <td><?php echo empty($last_sync) ? 'Never synced' : $last_sync; ?></td>
                </tr>
            </table>
            <?php
            submit_button();
            ?>
        </form>
        </div><?php
    }
}

$GLOBALS['gtcsync_settings'] = new GTCSyncSettings();